@@ -53,9 +53,6 @@
 /* got in_addr_t? */
 #undef HAVE_IN_ADDR_T
 
-/* got ssize_t? */
-#undef HAVE_SSIZE_T
-
 /* define if you have getdevs() */
 #undef HAVE_GETDEVS
 
@@ -85,18 +82,12 @@
 /* define if you have gethostbyname */
 #undef HAVE_GETHOSTBYNAME
 
-/* define if you have getservbyname */
-#undef HAVE_GETSERVBYNAME
-
 /* define if you have the perl_eval_pv() function */
 #undef HAVE_PERL_EVAL_PV_LC
 
 /* define if you have the Perl_eval_pv() function */
 #undef HAVE_PERL_EVAL_PV_UC
 
-/* Define to 1 if you have the `perl_eval_pv' function. */
-#undef HAVE_PERL_EVAL_PV
-
 /* printing system */
 #undef HAVE_LPSTAT
 #undef LPSTAT_PATH
@@ -567,7 +558,7 @@
 #ifdef WIN32
 #define NETSNMP_OSTYPE NETSNMP_WIN32ID
 #endif
-#if defined(aix3) || defined(aix4) || defined(aix5)
+#if defined(aix3) || defined(aix4) || defined(aix5) || defined(aix6)
 #define NETSNMP_OSTYPE NETSNMP_AIXID
 #endif
 #if defined(darwin) && (darwin >= 8)
@@ -767,7 +758,6 @@
  *    static NETSNMP_INLINE function(int parm) { return parm -1; }
  *
  */
-#undef NETSNMP_BROKEN_INLINE
 #ifdef NETSNMP_BROKEN_INLINE
 #   define NETSNMP_ENABLE_INLINE 0
 #else
