@@ -363,7 +363,7 @@ netsnmp_register_int_instance_context(const char *name,
     myreg = get_reg(name, "int_handler", reg_oid, reg_oid_len, it,
                     HANDLER_CAN_RWRITE, netsnmp_instance_int_handler,
                     subhandler, contextName);
-    return netsnmp_register_read_only_instance(myreg);
+    return netsnmp_register_instance(myreg);
 }
 
 int
@@ -589,7 +589,7 @@ netsnmp_instance_long_handler(netsnmp_mib_handler *handler,
                               netsnmp_request_info *requests)
 {
 
-    long           *it = (u_long *) handler->myvoid;
+    long           *it = (long *) handler->myvoid;
     long           *it_save;
 
     DEBUGMSGTL(("netsnmp_instance_long_handler", "Got request:  %d\n",
@@ -713,7 +713,7 @@ netsnmp_instance_int_handler(netsnmp_mib_handler *handler,
         /*
          * update current 
          */
-        DEBUGMSGTL(("testhandler", "updated int %d -> %l\n", *it,
+        DEBUGMSGTL(("testhandler", "updated int %d -> %ld\n", *it,
                     *(requests->requestvb->val.integer)));
         *it = (int) *(requests->requestvb->val.integer);
         break;
@@ -827,7 +827,7 @@ netsnmp_instance_num_file_handler(netsnmp_mib_handler *handler,
         /*
          * update current 
          */
-        DEBUGMSGTL(("helper:instance", "updated %s -> %l\n", nfi->file_name,
+        DEBUGMSGTL(("helper:instance", "updated %s -> %ld\n", nfi->file_name,
                     *(requests->requestvb->val.integer)));
         it = *(requests->requestvb->val.integer);
         rewind(nfi->filep); /* rewind to make sure we are at the beginning */
@@ -922,7 +922,7 @@ netsnmp_instance_uint_handler(netsnmp_mib_handler *handler,
         /*
          * update current 
          */
-        DEBUGMSGTL(("testhandler", "updated uint %d -> %l\n", *it,
+        DEBUGMSGTL(("testhandler", "updated uint %d -> %ld\n", *it,
                     *(requests->requestvb->val.integer)));
         *it = (unsigned int) *(requests->requestvb->val.integer);
         break;
@@ -962,7 +962,7 @@ netsnmp_instance_helper_handler(netsnmp_mib_handler *handler,
                            requests->requestvb->name_length,
                            reginfo->rootoid, reginfo->rootoid_len);
 
-    DEBUGMSGTL(("helper:instance", "  oid:", cmp));
+    DEBUGMSGTL(("helper:instance", "  oid:"));
     DEBUGMSGOID(("helper:instance", var->name, var->name_length));
     DEBUGMSG(("helper:instance", "\n"));
 
