@@ -117,7 +117,7 @@ mteTrigger_createEntry(char *mteOwner, char *mteTName, int fixed)
     netsnmp_table_row_add_index(row, ASN_PRIV_IMPLIED_OCTET_STR,
                                 entry->mteTName, mteTName_len);
 
- /* entry->mteTriggerTest         = MTE_TRIGGER_BOOLEAN; */
+  /* entry->mteTriggerTest         = MTE_TRIGGER_BOOLEAN; */
     entry->mteTriggerValueID_len  = 2;  /* .0.0 */
     entry->mteTriggerFrequency    = 600;
     memcpy(entry->mteDeltaDiscontID, _sysUpTime_instance,
@@ -210,6 +210,23 @@ mteTrigger_run( unsigned int reg, void *clientarg)
         return;
     }
 
+    {
+	extern netsnmp_agent_session *netsnmp_processing_set;
+	if (netsnmp_processing_set) {
+	    /*
+	     * netsnmp_handle_request will not be responsive to our efforts to
+	     *	Retrieve the requested MIB value(s)...
+	     * so we will skip it.
+	     * https://sourceforge.net/tracker/
+	     *	index.php?func=detail&aid=1557406&group_id=12694&atid=112694
+	     */
+	    DEBUGMSGTL(("disman:event:trigger:monitor",
+		"Skipping trigger (%s) while netsnmp_processing_set\n",
+		entry->mteTName));
+	    return;
+	}
+    }
+
     /*
      * Retrieve the requested MIB value(s)...
      */
@@ -319,7 +336,7 @@ mteTrigger_run( unsigned int reg, void *clientarg)
                     if (vp1_prev) {
                         vp1_prev->next_variable = vtmp;
                     } else {
-                        entry->old_results      = vtmp;
+                        var                     = vtmp;
                     }
                     vp1_prev = vtmp;
                     vp2_prev = vp2;
@@ -416,7 +433,7 @@ mteTrigger_run( unsigned int reg, void *clientarg)
              */
             if (entry->mteTExTest & entry->mteTExStartup & MTE_EXIST_ABSENT) {
                 if (!(entry->flags & MTE_TRIGGER_FLAG_VWILD) &&
-                    vp1->type == ASN_NULL ) {
+                    var->type == ASN_NULL ) {
                     DEBUGMSGTL(( "disman:event:trigger:fire",
                                  "Firing initial existence test: "));
                     DEBUGMSGOID(("disman:event:trigger:fire",
@@ -432,10 +449,10 @@ mteTrigger_run( unsigned int reg, void *clientarg)
                      *  to report a NULL value, but this clashes with
                      * the syntax of the mteHotValue MIB object.
                      */
-                    entry->mteTriggerFired    = vp1;
+                    entry->mteTriggerFired    = var;
                     n = entry->mteTriggerValueID_len;
                     mteEvent_fire(entry->mteTExEvOwner, entry->mteTExEvent, 
-                                  entry, vp1->name+n, vp1->name_length-n);
+                                  entry, var->name+n, var->name_length-n);
                 }
             }
         } /* !old_results */
@@ -530,7 +547,7 @@ mteTrigger_run( unsigned int reg, void *clientarg)
         case ASN_TIMETICKS:
         case ASN_UINTEGER:
         case ASN_COUNTER64:
-#ifdef OPAQUE_SPECIAL_TYPES
+#ifdef NETSNMP_WITH_OPAQUE_SPECIAL_TYPES
         case ASN_OPAQUE_COUNTER64:
         case ASN_OPAQUE_U64:
         case ASN_OPAQUE_I64:
@@ -592,9 +609,9 @@ mteTrigger_run( unsigned int reg, void *clientarg)
                 snmp_set_var_objid( dvar, entry->mteDeltaDiscontID,
                                           entry->mteDeltaDiscontID_len );
                 if ( entry->flags & MTE_TRIGGER_FLAG_DWILD ) {
-                    netsnmp_query_walk( dvar, entry->session );
+                    n = netsnmp_query_walk( dvar, entry->session );
                 } else {
-                    netsnmp_query_get(  dvar, entry->session );
+                    n = netsnmp_query_get(  dvar, entry->session );
                 }
                 if ( n != SNMP_ERR_NOERROR ) {
                     _mteTrigger_failure( "failed to run mteTrigger delta query" );
@@ -826,16 +843,22 @@ mteTrigger_run( unsigned int reg, void *clientarg)
              *     to remember whether the trigger has already fired)
              */
             if ( cmp ) {
-                if ((entry->old_results ||
-                     (entry->flags & MTE_TRIGGER_FLAG_BSTART)) &&
-                    (vp1->index & MTE_ARMED_BOOLEAN )) {
+              if (vp1->index & MTE_ARMED_BOOLEAN ) {
+                vp1->index &= ~MTE_ARMED_BOOLEAN;
+                /*
+                 * NB: Clear the trigger armed flag even if the
+                 *   (starting) event dosn't actually fire.
+                 *   Otherwise initially true (but suppressed)
+                 *   triggers will fire on the *second* probe.
+                 */
+                if ( entry->old_results ||
+                    (entry->flags & MTE_TRIGGER_FLAG_BSTART)) {
                     DEBUGMSGTL(( "disman:event:trigger:fire",
                                  "Firing boolean test: "));
                     DEBUGMSGOID(("disman:event:trigger:fire",
                                  vp1->name, vp1->name_length));
                     DEBUGMSG((   "disman:event:trigger:fire", "%s\n",
                                   (entry->old_results ? "" : " (startup)")));
-                    vp1->index &= ~MTE_ARMED_BOOLEAN;
                     entry->mteTriggerXOwner   = entry->mteTBoolObjOwner;
                     entry->mteTriggerXObjects = entry->mteTBoolObjects;
                     /*
@@ -848,6 +871,7 @@ mteTrigger_run( unsigned int reg, void *clientarg)
                     mteEvent_fire(entry->mteTBoolEvOwner, entry->mteTBoolEvent, 
                                   entry, vp1->name+n, vp1->name_length-n);
                 }
+              }
             } else {
                 vp1->index |= MTE_ARMED_BOOLEAN;
             }
@@ -923,17 +947,24 @@ mteTrigger_run( unsigned int reg, void *clientarg)
              */
             cmp = vp1->index;   /* working copy of 'armed' flags */
             if ( value >= entry->mteTThRiseValue ) {
-                if ((entry->old_results ||
-                     (entry->mteTThStartup & MTE_THRESH_START_RISE)) &&
-                    (vp1->index & MTE_ARMED_TH_RISE )) {
+              if (cmp & MTE_ARMED_TH_RISE ) {
+                cmp &= ~MTE_ARMED_TH_RISE;
+                cmp |=  MTE_ARMED_TH_FALL;
+                /*
+                 * NB: Clear the trigger armed flag even if the
+                 *   (starting) event dosn't actually fire.
+                 *   Otherwise initially true (but suppressed)
+                 *   triggers will fire on the *second* probe.
+                 * Similarly for falling thresholds (see below).
+                 */
+                if ( entry->old_results ||
+                    (entry->mteTThStartup & MTE_THRESH_START_RISE)) {
                     DEBUGMSGTL(( "disman:event:trigger:fire",
                                  "Firing rising threshold test: "));
                     DEBUGMSGOID(("disman:event:trigger:fire",
                                  vp1->name, vp1->name_length));
                     DEBUGMSG((   "disman:event:trigger:fire", "%s\n",
                                  (entry->old_results ? "" : " (startup)")));
-                    cmp &= ~MTE_ARMED_TH_RISE;
-                    cmp |=  MTE_ARMED_TH_FALL;
                     /*
                      * If no riseEvent is configured, we need still to
                      *  set the armed flags appropriately, but there's
@@ -949,20 +980,22 @@ mteTrigger_run( unsigned int reg, void *clientarg)
                                       entry, vp1->name+n, vp1->name_length-n);
                     }
                 }
+              }
             }
 
             if ( value <= entry->mteTThFallValue ) {
-                if ((entry->old_results ||
-                     (entry->mteTThStartup & MTE_THRESH_START_FALL)) &&
-                    (vp1->index & MTE_ARMED_TH_FALL )) {
+              if (cmp & MTE_ARMED_TH_FALL ) {
+                cmp &= ~MTE_ARMED_TH_FALL;
+                cmp |=  MTE_ARMED_TH_RISE;
+                /* Clear the trigger armed flag (see above) */
+                if ( entry->old_results ||
+                    (entry->mteTThStartup & MTE_THRESH_START_FALL)) {
                     DEBUGMSGTL(( "disman:event:trigger:fire",
                                  "Firing falling threshold test: "));
                     DEBUGMSGOID(("disman:event:trigger:fire",
                                  vp1->name, vp1->name_length));
                     DEBUGMSG((   "disman:event:trigger:fire", "%s\n",
                                  (entry->old_results ? "" : " (startup)")));
-                    cmp &= ~MTE_ARMED_TH_FALL;
-                    cmp |=  MTE_ARMED_TH_RISE;
                     /*
                      * Similarly, if no fallEvent is configured,
                      *  there's no point in trying to fire it either.
@@ -977,6 +1010,7 @@ mteTrigger_run( unsigned int reg, void *clientarg)
                                       entry, vp1->name+n, vp1->name_length-n);
                     }
                 }
+              }
             }
             vp1->index = cmp;
         }
