@@ -406,7 +406,23 @@ var_hrswinst(struct variable * vp,
         break;
     case HRSWINST_NAME:
         {
+#ifdef HAVE_PKGINFO
+            char *pver;
+# endif
             strncpy(string, swi->swi_name, sizeof(string) - 1);
+
+/* If we are on a solaris machine, the package names do not include versioning info,
+ * so we must add it manually
+ */
+#ifdef HAVE_PKGINFO
+            pver = pkgparam(swi->swi_name, "VERSION");
+            /* 1 spot for the terminating null and one for the dash */
+            if (strlen(pver) + 2 + strlen(string) <= sizeof(string)) {
+                strcat(string, "-");
+                strcat(string, pver);
+            }
+# endif
+
             /*
              * This will be unchanged from the initial "null"
              * value, if swi->swi_name is not defined 
@@ -441,7 +457,19 @@ var_hrswinst(struct variable * vp,
                 free(catg);
             }
 #else
+# ifdef HAVE_LIBRPM
+            char *rpm_groups;
+            if ( headerGetEntry(swi->swi_h, RPMTAG_GROUP, NULL, (void **) &rpm_groups, NULL) ) {
+                if ( strstr(rpm_groups, "System Environment") != NULL )
+                    long_return = 2;	/* operatingSystem */
+                else
+                    long_return = 4;	/* applcation */
+            } else {
+                long_return = 1;    /* unknown */
+            }
+# else
             long_return = 1;    /* unknown */
+# endif
 #endif
             ret = (u_char *) & long_return;
         }
@@ -450,9 +478,7 @@ var_hrswinst(struct variable * vp,
         {
 #ifdef HAVE_LIBRPM
             int_32         *rpm_data;
-            headerGetEntry(swi->swi_h, RPMTAG_INSTALLTIME, NULL,
-                           (void **) &rpm_data, NULL);
-            if (rpm_data != NULL) {
+            if ( headerGetEntry(swi->swi_h, RPMTAG_INSTALLTIME, NULL, (void **) &rpm_data, NULL) ) {
                 time_t          installTime = *rpm_data;
                 ret = date_n_time(&installTime, var_len);
             } else {
