@@ -1,11 +1,16 @@
 /*
  * interface data access header
  *
- * $Id: interface.h,v 1.11 2005/10/28 15:29:22 rstory Exp $
+ * $Id: interface.h 16536 2007-06-18 21:13:46Z hardaker $
  */
 #ifndef NETSNMP_ACCESS_INTERFACE_CONFIG_H
 #define NETSNMP_ACCESS_INTERFACE_CONFIG_H
 
+/*
+ * all platforms use this generic code
+ */
+config_require(if-mib/data_access/interface)
+
 /**---------------------------------------------------------------------*/
 /*
  * configure required files
@@ -20,10 +25,30 @@
  *    HPUX11 had different reuirements than other HPUX, that should
  *    be handled in the *_hpux.h header file.
  */
-config_require(if-mib/data_access/interface)
-#if defined( linux )
-config_require(if-mib/data_access/interface_linux)
-config_require(if-mib/data_access/interface_ioctl)
+
+#ifdef NETSNMP_INCLUDE_IFTABLE_REWRITES
+
+config_exclude(mibII/interfaces)
+
+#   if defined( linux )
+
+    config_require(if-mib/data_access/interface_linux)
+    config_require(if-mib/data_access/interface_ioctl)
+
+#   elif defined( openbsd3 ) || defined( openbsd4 ) || \
+    defined( freebsd4 ) || defined( freebsd5 ) || defined( freebsd6 ) || defined (darwin)
+
+    config_require(if-mib/data_access/interface_sysctl)
+
+#   elif defined( solaris2 )
+
+    config_require(if-mib/data_access/interface_solaris2)
+
+#   else
+
+    config_error(This platform does not yet support IF-MIB rewrites)
+
+#   endif
 #else
 #   define NETSNMP_ACCESS_INTERFACE_NOARCH 1
 #endif
