@@ -1,10 +1,20 @@
 /*
  *  Interface MIB architecture support
  *
- * $Id: interface_linux.c,v 1.28.2.4 2006/04/25 17:39:54 hardaker Exp $
+ * $Id: interface_linux.c 16944 2008-05-14 13:35:40Z tanders $
  */
 #include <net-snmp/net-snmp-config.h>
 #include <net-snmp/net-snmp-includes.h>
+
+#ifdef HAVE_LINUX_ETHTOOL_H
+#include <linux/types.h>
+typedef __u64 u64;         /* hack, so we may include kernel's ethtool.h */
+typedef __u32 u32;         /* ditto */
+typedef __u16 u16;         /* ditto */
+typedef __u8 u8;           /* ditto */
+#include <linux/ethtool.h>
+#endif /* HAVE_LINUX_ETHTOOL_H */
+
 #include "mibII/mibII_common.h"
 #include "if-mib/ifTable/ifTable_constants.h"
 
@@ -22,17 +32,15 @@
 #include "interface_ioctl.h"
 
 #include <sys/types.h>
+#include <sys/stat.h>
+#include <unistd.h>
 
-#ifdef HAVE_LINUX_ETHTOOL_H
-#include <sys/types.h>
-typedef unsigned long long u64;         /* hack, so we may include kernel's ethtool.h */
-typedef __uint32_t u32;         /* ditto */
-typedef __uint16_t u16;         /* ditto */
-typedef __uint8_t u8;           /* ditto */
-#include <linux/ethtool.h>
-#endif /* HAVE_LINUX_ETHTOOL_H */
 #include <linux/sockios.h>
 
+#ifndef IF_NAMESIZE
+#define IF_NAMESIZE 16
+#endif
+
 unsigned int
 netsnmp_linux_interface_get_if_speed(int fd, const char *name);
 #ifdef HAVE_LINUX_ETHTOOL_H
@@ -40,13 +48,48 @@ unsigned int
 netsnmp_linux_interface_get_if_speed_mii(int fd, const char *name);
 #endif
 
+#define PROC_SYS_NET_IPVx_NEIGH_RETRANS_TIME_MS "/proc/sys/net/ipv%d/neigh/%s/retrans_time_ms"
+#define PROC_SYS_NET_IPVx_NEIGH_RETRANS_TIME    "/proc/sys/net/ipv%d/neigh/%s/retrans_time"
+static const char *proc_sys_retrans_time;
+static unsigned short retrans_time_factor = 1;
 
+
+#define PROC_SYS_NET_IPVx_BASE_REACHABLE_TIME_MS "/proc/sys/net/ipv%d/neigh/%s/base_reachable_time_ms"
+#define PROC_SYS_NET_IPVx_BASE_REACHABLE_TIME "/proc/sys/net/ipv%d/neigh/%s/base_reachable_time"
+static const char *proc_sys_basereachable_time;
+static unsigned short basereachable_time_ms = 0;
 void
 netsnmp_arch_interface_init(void)
 {
     /*
-     * nothing to do
+     * Check which retransmit time interface is available
      */
+    char proc_path[ 64+IF_NAMESIZE];
+    char proc_path2[64+IF_NAMESIZE];
+    struct stat st;
+
+    snprintf(proc_path,  sizeof(proc_path),
+             PROC_SYS_NET_IPVx_NEIGH_RETRANS_TIME_MS, 6, "default");
+    snprintf(proc_path2, sizeof(proc_path2),
+             PROC_SYS_NET_IPVx_NEIGH_RETRANS_TIME_MS, 4, "default");
+
+    if ((stat(proc_path, &st) == 0) || (stat(proc_path2, &st) == 0)) {
+        proc_sys_retrans_time = PROC_SYS_NET_IPVx_NEIGH_RETRANS_TIME_MS;
+    } else {
+        proc_sys_retrans_time = PROC_SYS_NET_IPVx_NEIGH_RETRANS_TIME;
+        retrans_time_factor = 10;
+    }
+
+    snprintf(proc_path,  sizeof(proc_path),  PROC_SYS_NET_IPVx_BASE_REACHABLE_TIME_MS, 6, "default");
+    snprintf(proc_path2,  sizeof(proc_path),  PROC_SYS_NET_IPVx_BASE_REACHABLE_TIME, 4, "default");
+
+    if ((stat(proc_path, &st) == 0) || (stat(proc_path2, &st) == 0)) {
+        proc_sys_basereachable_time = PROC_SYS_NET_IPVx_BASE_REACHABLE_TIME_MS;
+        basereachable_time_ms = 1;
+    }
+    else {
+        proc_sys_basereachable_time = PROC_SYS_NET_IPVx_BASE_REACHABLE_TIME;
+    }
 }
 
 /*
@@ -72,7 +115,7 @@ void
 _arch_interface_has_ipv6(oid if_index, u_int *flags,
                          netsnmp_container *addr_container)
 {
-#ifdef INET6
+#ifdef NETSNMP_ENABLE_IPV6
     netsnmp_ipaddress_entry *addr_entry = NULL;
     netsnmp_iterator        *addr_it = NULL;
     u_int                    addr_container_flags = 0; /* must init to 0 */
@@ -83,7 +126,15 @@ _arch_interface_has_ipv6(oid if_index, u_int *flags,
 
     *flags &= ~NETSNMP_INTERFACE_FLAGS_HAS_IPV6;
 
-#ifdef INET6
+#ifndef SIOCGMIIPHY
+#define SIOCGMIIPHY 0x8947
+#endif
+
+#ifndef SIOCGMIIREG
+#define SIOCGMIIREG 0x8948
+#endif
+
+#ifdef NETSNMP_ENABLE_IPV6
     /*
      * get ipv6 addresses
      */
@@ -161,7 +212,7 @@ _arch_interface_flags_v4_get(netsnmp_interface_entry *entry)
     /*
      * get the retransmit time
      */
-    snprintf(line,sizeof(line),"/proc/sys/net/ipv4/neigh/%s/retrans_time",
+    snprintf(line,sizeof(line), proc_sys_retrans_time, 4,
              entry->name);
     if (!(fin = fopen(line, "r"))) {
         DEBUGMSGTL(("access:interface",
@@ -169,7 +220,7 @@ _arch_interface_flags_v4_get(netsnmp_interface_entry *entry)
     }
     else {
         if (fgets(line, sizeof(line), fin)) {
-            entry->retransmit_v4 = atoi(line) * 100;
+            entry->retransmit_v4 = atoi(line) * retrans_time_factor;
             entry->ns_flags |= NETSNMP_INTERFACE_FLAGS_HAS_V4_RETRANSMIT;
         }
         fclose(fin);
@@ -177,7 +228,7 @@ _arch_interface_flags_v4_get(netsnmp_interface_entry *entry)
 }
 
 
-#ifdef INET6
+#ifdef NETSNMP_ENABLE_IPV6
 /**
  * @internal
  */
@@ -190,7 +241,7 @@ _arch_interface_flags_v6_get(netsnmp_interface_entry *entry)
     /*
      * get the retransmit time
      */
-    snprintf(line,sizeof(line),"/proc/sys/net/ipv6/neigh/%s/retrans_time",
+    snprintf(line,sizeof(line), proc_sys_retrans_time, 6,
              entry->name);
     if (!(fin = fopen(line, "r"))) {
         DEBUGMSGTL(("access:interface",
@@ -198,7 +249,7 @@ _arch_interface_flags_v6_get(netsnmp_interface_entry *entry)
     }
     else {
         if (fgets(line, sizeof(line), fin)) {
-            entry->retransmit_v6 = atoi(line);
+            entry->retransmit_v6 = atoi(line) * retrans_time_factor;
             entry->ns_flags |= NETSNMP_INTERFACE_FLAGS_HAS_V6_RETRANSMIT;
         }
         fclose(fin);
@@ -207,7 +258,7 @@ _arch_interface_flags_v6_get(netsnmp_interface_entry *entry)
     /*
      * get the forwarding status
      */
-    snprintf(line,sizeof(line),"/proc/sys/net/ipv6/conf/%s/forwarding",
+    snprintf(line, sizeof(line), "/proc/sys/net/ipv6/conf/%s/forwarding",
              entry->name);
     if (!(fin = fopen(line, "r"))) {
         DEBUGMSGTL(("access:interface",
@@ -224,21 +275,25 @@ _arch_interface_flags_v6_get(netsnmp_interface_entry *entry)
     /*
      * get the reachable time
      */
-    snprintf(line,sizeof(line),"/proc/sys/net/ipv6/neigh/%s/base_reachable_time",
-             entry->name);
+    snprintf(line, sizeof(line), proc_sys_basereachable_time, 6, entry->name);
     if (!(fin = fopen(line, "r"))) {
         DEBUGMSGTL(("access:interface",
                     "Failed to open %s\n", line));
     }
     else {
         if (fgets(line, sizeof(line), fin)) {
-            entry->reachable_time = atoi(line) * 1000; /* sec to millisec */
+            if (basereachable_time_ms) {
+                entry->reachable_time = atoi(line); /* millisec */
+            } else {
+                entry->reachable_time = atoi(line)*1000; /* sec to  millisec */
+            }
+
             entry->ns_flags |= NETSNMP_INTERFACE_FLAGS_HAS_V6_REACHABLE;
         }
         fclose(fin);
     }
 }
-#endif /* INET6 */
+#endif /* NETSNMP_ENABLE_IPV6 */
 
 /**
  * @internal
@@ -286,6 +341,11 @@ _parse_stats(netsnmp_interface_entry *entry, char *stats, int expected)
     while (*stats == ' ')
         stats++;
 
+    if ((*stats == 'N') &&
+        (0 == strncmp(stats, "No statistics available",
+                      sizeof("No statistics available"))))
+        return -1;
+
     /*
      * Now parse the rest of the line (i.e. starting from 'stats')
      *      to extract the relevant statistics, and populate
@@ -326,7 +386,6 @@ _parse_stats(netsnmp_interface_entry *entry, char *stats, int expected)
         snmp_log(LOG_ERR,
                  "error scanning interface data (expected %d, got %d)\n",
                  expected, scan_count);
-        netsnmp_access_interface_entry_free(entry);
         return scan_count;
     }
     entry->ns_flags |= NETSNMP_INTERFACE_FLAGS_ACTIVE;
@@ -363,11 +422,13 @@ _parse_stats(netsnmp_interface_entry *entry, char *stats, int expected)
     
     /*
      * calculated stats.
-         *
-         *  we have imcast, but not ibcast.
-         */
+     *
+     *  we have imcast, but not ibcast.
+     */
     entry->stats.inucast = entry->stats.imcast.low +
         entry->stats.ibcast.low;
+    entry->stats.onucast = entry->stats.omcast.low +
+        entry->stats.obcast.low;
     
     return 0;
 }
@@ -388,7 +449,7 @@ netsnmp_arch_interface_container_load(netsnmp_container* container,
     netsnmp_interface_entry *entry = NULL;
     static char     scan_expected = 0;
     int             fd;
-#ifdef INET6
+#ifdef NETSNMP_ENABLE_IPV6
     netsnmp_container *addr_container;
 #endif
 
@@ -416,7 +477,7 @@ netsnmp_arch_interface_container_load(netsnmp_container* container,
         return -2;
     }
 
-#ifdef INET6
+#ifdef NETSNMP_ENABLE_IPV6
     /*
      * get ipv6 addresses
      */
@@ -454,7 +515,7 @@ netsnmp_arch_interface_container_load(netsnmp_container* container,
      */
     while (fgets(line, sizeof(line), devin)) {
         char           *stats, *ifstart = line;
-        int             flags;
+        u_int           flags;
         oid             if_index;
 
         flags = 0;
@@ -492,7 +553,7 @@ netsnmp_arch_interface_container_load(netsnmp_container* container,
          * ip version is to look for ip addresses. If anyone
          * knows a better way, put it here!
          */
-#ifdef INET6
+#ifdef NETSNMP_ENABLE_IPV6
         _arch_interface_has_ipv6(if_index, &flags, addr_container);
 #endif
         netsnmp_access_interface_ioctl_has_ipv4(fd, ifstart, 0, &flags);
@@ -512,7 +573,7 @@ netsnmp_arch_interface_container_load(netsnmp_container* container,
 
         entry = netsnmp_access_interface_entry_create(ifstart, 0);
         if(NULL == entry) {
-#ifdef INET6
+#ifdef NETSNMP_ENABLE_IPV6
             netsnmp_access_ipaddress_container_free(addr_container, 0);
 #endif
             netsnmp_access_interface_container_free(container,
@@ -612,7 +673,10 @@ netsnmp_arch_interface_container_load(netsnmp_container* container,
          * hardcoded max packet size
          * (see ip_frag_reasm: if(len > 65535) goto out_oversize;)
          */
-        entry->reasm_max = 65535;
+        entry->reasm_max_v4 = entry->reasm_max_v6 = 65535;
+        entry->ns_flags |= 
+            NETSNMP_INTERFACE_FLAGS_HAS_V4_REASMMAX |
+            NETSNMP_INTERFACE_FLAGS_HAS_V6_REASMMAX;
 
         netsnmp_access_interface_entry_overrides(entry);
 
@@ -624,17 +688,17 @@ netsnmp_arch_interface_container_load(netsnmp_container* container,
         if (flags & NETSNMP_INTERFACE_FLAGS_HAS_IPV4)
             _arch_interface_flags_v4_get(entry);
 
-#ifdef INET6
+#ifdef NETSNMP_ENABLE_IPV6
         if (flags & NETSNMP_INTERFACE_FLAGS_HAS_IPV6)
             _arch_interface_flags_v6_get(entry);
-#endif /* INET6 */
+#endif /* NETSNMP_ENABLE_IPV6 */
 
         /*
          * add to container
          */
         CONTAINER_INSERT(container, entry);
     }
-#ifdef INET6
+#ifdef NETSNMP_ENABLE_IPV6
     netsnmp_access_ipaddress_container_free(addr_container, 0);
 #endif
     fclose(devin);
@@ -714,7 +778,6 @@ netsnmp_linux_interface_get_if_speed(int fd, const char *name)
      */
     ushort *data = (ushort *)(&ifr.ifr_data);
     unsigned phy_id;
-    unsigned char new_ioctl_nums = 0;
     int mii_reg, i;
     ushort mii_val[32];
     ushort bmcr, bmsr, nway_advert, lkpar;
@@ -725,11 +788,11 @@ netsnmp_linux_interface_get_if_speed(int fd, const char *name)
     ifr.ifr_name[ sizeof(ifr.ifr_name)-1 ] = 0;
     data[0] = 0;
     
-    if (ioctl(fd, 0x8947, &ifr) >= 0) {
-        new_ioctl_nums = 1;
-    } else if (ioctl(fd, SIOCDEVPRIVATE, &ifr) >= 0) {
-        new_ioctl_nums = 0;
-    } else {
+    /*
+     * SIOCGMIIPHY has been defined since at least kernel 2.4.10 (Sept 2001).
+     * It's probably safe to drop the interim SIOCDEVPRIVATE handling now!
+     */
+    if (ioctl(fd, SIOCGMIIPHY, &ifr) < 0) {
         DEBUGMSGTL(("mibII/interfaces", "SIOCGMIIPHY on %s failed\n",
                     ifr.ifr_name));
         return retspeed;
@@ -740,7 +803,7 @@ netsnmp_linux_interface_get_if_speed(int fd, const char *name)
     for (mii_reg = 0; mii_reg < 8; mii_reg++){
         data[0] = phy_id;
         data[1] = mii_reg;
-        if(ioctl(fd, new_ioctl_nums ? 0x8948 : SIOCDEVPRIVATE+1, &ifr) <0){
+        if(ioctl(fd, SIOCGMIIREG, &ifr) <0){
             DEBUGMSGTL(("mibII/interfaces", "SIOCGMIIREG on %s failed\n", ifr.ifr_name));
         }
         mii_val[mii_reg] = data[3];		
