@@ -3,7 +3,7 @@
  *
  *  Based on patch 1362403, submited by Rojer
  *
- * $Id: interface_sysctl.c,v 1.2 2006/09/27 17:54:10 rstory Exp $
+ * $Id: interface_sysctl.c 16552 2007-06-20 21:34:24Z tanders $
  */
 #include <net-snmp/net-snmp-config.h>
 #include <net-snmp/net-snmp-includes.h>
@@ -445,8 +445,10 @@ netsnmp_arch_interface_container_load(netsnmp_container* container,
             entry->oper_status = IFOPERSTATUS_DOWN;
         }
 
-        entry->ns_flags |= NETSNMP_INTERFACE_FLAGS_HAS_V4_REASMMAX;
         entry->reasm_max_v4 = entry->reasm_max_v6 = IP_MAXPACKET;
+        entry->ns_flags |= 
+            NETSNMP_INTERFACE_FLAGS_HAS_V4_REASMMAX |
+            NETSNMP_INTERFACE_FLAGS_HAS_V6_REASMMAX;
 
         /* get counters */
         entry->stats.ibytes.low = ifp->ifm_data.ifi_ibytes;
@@ -496,7 +498,7 @@ netsnmp_arch_interface_container_load(netsnmp_container* container,
         CONTAINER_INSERT(container, entry);
         DEBUGMSGTL(("access:interface:container:sysctl",
                     "created entry %u for %s\n", entry->index, entry->name));
-    } /* for (each interfac entry) */
+    } /* for (each interface entry) */
 
     /* pass 2: walk addresses */
     for (cp = if_list; cp < if_list + if_list_size; cp += ifa->ifam_msglen) {
