@@ -150,42 +150,40 @@ smux_parse_peer_auth(const char *token, char *cptr)
 	config_perror("Too many smuxpeers");
 	return;
     }
-    aptr->sa_active_fd = -1;
-    if (!cptr) {
-        /*
-         * null passwords OK 
-         */
-        Auths[nauths++] = aptr;
-        DEBUGMSGTL(("smux_conf", "null password\n"));
-        return;
-    }
 
     /*
      * oid 
      */
-    password_cptr = strchr(cptr, ' ');
-    cptr_len = strlen(cptr);
-    if (password_cptr)
-        *password_cptr = 0x0;
-
+    aptr->sa_active_fd = -1;
     aptr->sa_oid_len = MAX_OID_LEN;
     read_objid( cptr, aptr->sa_oid, &aptr->sa_oid_len );
-
     DEBUGMSGTL(("smux_conf", "parsing registration for: %s\n", cptr));
 
-    if ((&password_cptr - &cptr + 1) < cptr_len) {
-        cptr = ++password_cptr;
-        DEBUGMSGTL(("smux_conf", "password is: %s\n",
-                    SNMP_STRORNULL(cptr)));
+    password_cptr = strchr(cptr, ' ');
+    cptr_len = strlen(cptr);
+
+    if (password_cptr != NULL) {    /* Do we have a password or not? */
+        *password_cptr = 0x0;
+	if ((&password_cptr - &cptr + 1) < cptr_len) {
+	    cptr = ++password_cptr;
+	    DEBUGMSGTL(("smux_conf", "password is: %s\n",
+	                SNMP_STRORNULL(cptr)));
+	}
+
+        /*
+         * password 
+         */
+        if (cptr)
+            strcpy(aptr->sa_passwd, cptr);
+    } else {
+        /*
+         * null passwords OK 
+         */
+        DEBUGMSGTL(("smux_conf", "null password\n"));
     }
 
-    /*
-     * password 
-     */
-    if (cptr)
-        strcpy(aptr->sa_passwd, cptr);
-
     Auths[nauths++] = aptr;
+    return;
 }
 
 void
@@ -352,7 +350,7 @@ var_smux_write(int action,
     u_char          buf[SMUXMAXPKTSIZE], *ptr, sout[3], type;
     int             reterr;
     size_t          var_len, datalen, name_length, packet_len;
-    ssize_t         len;
+    ssize_t         len, tmp_len;
     long            reqid, errsts, erridx;
     u_char          var_type, *dataptr;
 
@@ -417,7 +415,7 @@ var_smux_write(int action,
             return SNMP_ERR_GENERR;
         }
 
-        if (send(rptr->sr_fd, buf, len, 0) < 0) {
+        if (sendto(rptr->sr_fd, buf, len, 0, NULL, 0) < 0) {
             DEBUGMSGTL(("smux", "[var_smux_write] send failed\n"));
             return SNMP_ERR_GENERR;
         }
@@ -426,14 +424,19 @@ var_smux_write(int action,
             /*
              * peek at what's received 
              */
-            if ((len = recv(rptr->sr_fd, buf,
-                            SMUXMAXPKTSIZE, MSG_PEEK)) <= 0) {
+            if ((len = recvfrom(rptr->sr_fd, buf,
+                            SMUXMAXPKTSIZE, MSG_PEEK, NULL, 0)) <= 0) {
+                if ((len == -1) && ((errno == EINTR) || (errno == EAGAIN)))
+                {
+                   continue;
+                }
                 DEBUGMSGTL(("smux",
                             "[var_smux_write] peek failed or timed out\n"));
                 /*
                  * do we need to do a peer cleanup in this case?? 
                  */
                 smux_peer_cleanup(rptr->sr_fd);
+                smux_snmp_select_list_del(rptr->sr_fd);
                 return SNMP_ERR_GENERR;
             }
 
@@ -457,13 +460,19 @@ var_smux_write(int action,
             /*
              * receive the first packet 
              */
-            if ((len = recv(rptr->sr_fd, buf, len, 0)) <= 0) {
+            tmp_len = len;
+            do
+            {
+               len = tmp_len;
+               len = recvfrom(rptr->sr_fd, buf, len, 0, NULL, 0);
+            }
+            while((len == -1) && ((errno == EINTR) || (errno == EAGAIN)));
+
+            if (len <= 0) {
                 DEBUGMSGTL(("smux",
                             "[var_smux_write] recv failed or timed out\n"));
-                /*
-                 * do we need to do a peer cleanup in this case?? 
-                 */
                 smux_peer_cleanup(rptr->sr_fd);
+                smux_snmp_select_list_del(rptr->sr_fd);
                 return SNMP_ERR_GENERR;
             }
 
@@ -539,7 +548,7 @@ var_smux_write(int action,
                         "[var_smux_write] entering FREE - sending Commit \n"));
         }
 
-        if ((send(rptr->sr_fd, sout, 3, 0)) < 0) {
+        if ((sendto(rptr->sr_fd, sout, 3, 0, NULL, 0)) < 0) {
             DEBUGMSGTL(("smux",
                         "[var_smux_write] send rollback/commit failed\n"));
             return SNMP_ERR_GENERR;
@@ -595,7 +604,13 @@ smux_accept(int sd)
         /*
          * now block for an OpenPDU 
          */
-        if ((length = recv(fd, (char *) data, SMUXMAXPKTSIZE, 0)) <= 0) {
+        do
+        {
+           length = recvfrom(fd, (char *) data, SMUXMAXPKTSIZE, 0, NULL, 0);
+        }
+        while((length == -1) && ((errno == EINTR) || (errno == EAGAIN)));
+
+        if (length <= 0) {
             DEBUGMSGTL(("smux",
                         "[smux_accept] peer on fd %d died or timed out\n",
                         fd));
@@ -610,14 +625,14 @@ smux_accept(int sd)
         if ((ptr = asn_parse_header(ptr, &len, &type)) == NULL) {
             smux_send_close(fd, SMUXC_PACKETFORMAT);
             close(fd);
-            DEBUGMSGTL(("smux", "[smux_accept] peer on %d sent bad open"));
+            DEBUGMSGTL(("smux", "[smux_accept] peer on %d sent bad open", fd));
             return -1;
         } else if (type != (u_char) SMUX_OPEN) {
             smux_send_close(fd, SMUXC_PROTOCOLERROR);
             close(fd);
             DEBUGMSGTL(("smux",
                         "[smux_accept] peer on %d did not send open: (%d)\n",
-                        type));
+                        fd, type));
             return -1;
         }
         ptr = smux_open_process(fd, ptr, &len, &fail);
@@ -662,10 +677,45 @@ smux_accept(int sd)
 int
 smux_process(int fd)
 {
-    int             length;
+    int             length, tmp_length;
     u_char          data[SMUXMAXPKTSIZE];
+    u_char          type, *ptr;
+    size_t          packet_len;
+
+    do
+    {
+       length = recvfrom(fd, (char *) data, SMUXMAXPKTSIZE, MSG_PEEK, NULL, 0);
+    }
+    while((length == -1) && ((errno == EINTR) || (errno == EAGAIN)));
+
+    if (length <= 0)
+    {
+       snmp_log_perror("[smux_process] peek failed");
+       smux_peer_cleanup(fd);
+       return -1;
+    }
+
+    /*
+     * determine if we received more than one packet 
+     */
+    packet_len = length;
+    ptr = asn_parse_header(data, &packet_len, &type);
+    packet_len += (ptr - data);
+    if (length > packet_len) {
+        /*
+         * set length to receive only the first packet 
+         */
+        length = packet_len;
+    }
+
+    tmp_length = length;
+    do
+    {
+       length = tmp_length;
+       length = recvfrom(fd, (char *) data, length, 0, NULL, 0);
+    }
+    while((length == -1) && ((errno == EINTR) || (errno == EAGAIN)));
 
-    length = recv(fd, (char *) data, SMUXMAXPKTSIZE, 0);
     if (length <= 0) {
         /*
          * the peer went away, close this descriptor 
@@ -731,14 +781,19 @@ smux_pdu_process(int fd, u_char * data, size_t length)
             break;
         case SMUX_TRAP:
             snmp_log(LOG_INFO, "Got trap from peer on fd %d\n", fd);
-            ptr = smux_trap_process(ptr, &len);
+            if (ptr != 0)
+            {
+               DEBUGMSGTL(("smux", "[smux_pdu_process] call smux_trap_process.\n"));
+               ptr = smux_trap_process(ptr, &len);
+            }
+            else
+            {
+               DEBUGMSGTL(("smux", "[smux_pdu_process] smux_trap_process not called: ptr=NULL.\n"));
+               DEBUGMSGTL(("smux", "[smux_pdu_process] Error: \n%s\n", snmp_api_errstring(0)));
+            }
             /*
              * watch out for close on top of this...should return correct end 
              */
-            /*
-             * debug this... 
-             */
-            ptr = NULL;
             break;
         default:
             smux_send_close(fd, SMUXC_PACKETFORMAT);
@@ -857,7 +912,7 @@ smux_send_close(int fd, int reason)
     /*
      * send a response back 
      */
-    if (send(fd, (char *) outpacket, 3, 0) < 0) {
+    if (sendto(fd, (char *) outpacket, 3, 0, NULL, 0) < 0) {
         snmp_log_perror("[smux_snmp_close] send failed");
     }
 }
@@ -1007,7 +1062,7 @@ smux_rreq_process(int sd, u_char * ptr, size_t * len)
                (operation == SMUX_REGOP_REGISTER_RW)) {
         if (priority < -1) {
             DEBUGMSGTL(("smux",
-                        "[smux_rreq_process] peer fd %d invalid priority",
+                        "[smux_rreq_process] peer fd %d invalid priority %ld",
                         sd, priority));
             smux_send_rrsp(sd, -1);
             return NULL;
@@ -1174,7 +1229,7 @@ smux_list_detach(smux_reg ** head, smux_reg ** m_remove)
 static int
 smux_list_add(smux_reg ** head, smux_reg * add)
 {
-    smux_reg       *rptr;
+    smux_reg       *rptr, *prev;
     int             result;
 
     if (*head == NULL) {
@@ -1182,6 +1237,7 @@ smux_list_add(smux_reg ** head, smux_reg * add)
         (*head)->sr_next = NULL;
         return 0;
     }
+    prev = NULL;
     for (rptr = *head; rptr->sr_next; rptr = rptr->sr_next) {
         result = snmp_oid_compare(add->sr_name, add->sr_name_len,
                                   rptr->sr_name, rptr->sr_name_len);
@@ -1207,15 +1263,22 @@ smux_list_add(smux_reg ** head, smux_reg * add)
             rptr->sr_next = add;
             return 0;
         }
+        prev = rptr;
     }
     /*
      * compare the last one 
      */
-    if ((snmp_oid_compare(add->sr_name, add->sr_name_len, rptr->sr_name,
-                          rptr->sr_name_len) == 0)
-        && add->sr_priority == rptr->sr_priority)
+    result = snmp_oid_compare(add->sr_name, add->sr_name_len, rptr->sr_name, rptr->sr_name_len);
+    if ((result == 0) && add->sr_priority == rptr->sr_priority)
         return -1;
-    else {
+    else  if (result < 0 ) {
+        add->sr_next = rptr;
+        if ( prev ) {
+            prev->sr_next = add;
+        } else {
+            *head = add;
+        }
+    } else {
         rptr->sr_next = add;
         add->sr_next = NULL;
     }
@@ -1265,6 +1328,7 @@ smux_snmp_process(int exact,
 {
     u_char          packet[SMUXMAXPKTSIZE], *ptr, result[SMUXMAXPKTSIZE];
     int             length = SMUXMAXPKTSIZE;
+    int             tmp_length;
     u_char          type;
     size_t          packet_len;
 
@@ -1287,7 +1351,7 @@ smux_snmp_process(int exact,
     DEBUGMSGOID(("smux", objid, *len));
     DEBUGMSG(("smux", "\n"));
 
-    if (send(sd, (char *) packet, length, 0) < 0) {
+    if (sendto(sd, (char *) packet, length, 0, NULL, 0) < 0) {
         snmp_log_perror("[smux_snmp_process] send failed");
     }
 
@@ -1299,11 +1363,19 @@ smux_snmp_process(int exact,
         /*
          * peek at what's received 
          */
-        length = recv(sd, (char *) result, SMUXMAXPKTSIZE, MSG_PEEK);
-        if (length < 0) {
-            snmp_log_perror("[smux_snmp_process] peek failed");
-            smux_peer_cleanup(sd);
-            return NULL;
+        length = recvfrom(sd, (char *) result, SMUXMAXPKTSIZE, MSG_PEEK, NULL, 0);
+        if (length <= 0) {
+            if ((length == -1) && ((errno == EINTR) || (errno == EAGAIN)))
+            {
+               continue;
+            }
+            else
+            {
+               snmp_log_perror("[smux_snmp_process] peek failed");
+               smux_peer_cleanup(sd);
+               smux_snmp_select_list_del(sd);
+               return NULL;
+            }
         }
 
         DEBUGMSGTL(("smux", "[smux_snmp_process] Peeked at %d bytes\n",
@@ -1326,11 +1398,19 @@ smux_snmp_process(int exact,
         /*
          * receive the first packet 
          */
-        length = recv(sd, (char *) result, length, 0);
-        if (length < 0) {
-            snmp_log_perror("[smux_snmp_process] recv failed");
-            smux_peer_cleanup(sd);
-            return NULL;
+        tmp_length = length;
+        do
+        {
+           length = tmp_length;
+           length = recvfrom(sd, (char *) result, length, 0, NULL, 0);
+        }
+        while((length == -1) && ((errno == EINTR) || (errno == EAGAIN)));
+
+        if (length <= 0) {
+           snmp_log_perror("[smux_snmp_process] recv failed");
+           smux_peer_cleanup(sd);
+           smux_snmp_select_list_del(sd);
+           return NULL;
         }
 
         DEBUGMSGTL(("smux", "[smux_snmp_process] Received %d bytes\n",
@@ -1725,7 +1805,7 @@ smux_send_rrsp(int sd, int pri)
         pri <<= 8;
     }
 
-    sent = send(sd, (char *) outdata, sizeof outdata, 0);
+    sent = sendto(sd, (char *) outdata, sizeof outdata, 0, NULL, 0);
     if (sent < 0) {
         DEBUGMSGTL(("smux", "[smux_send_rrsp] send failed\n"));
     }
@@ -1948,3 +2028,50 @@ smux_trap_process(u_char * rsp, size_t * len)
 
 }
 
+#define NUM_SOCKETS	32
+static int      sdlist[NUM_SOCKETS], sdlen = 0;
+
+int smux_snmp_select_list_add(int sd)
+{
+   if (sdlen < NUM_SOCKETS)
+   {
+      sdlist[sdlen++] = sd;
+      return(1);
+   }
+   return(0);
+}
+
+int smux_snmp_select_list_del(int sd)
+{
+   int i, found=0;
+
+   for (i = 0; i < (sdlen); i++) {
+      if (sdlist[i] == sd)
+      {
+         sdlist[i] = 0;
+         found = 1;
+      }
+      if ((found) &&(i < (sdlen - 1)))
+         sdlist[i] = sdlist[i + 1];
+   }
+   if (found)
+   {
+      sdlen--;
+      return(1);
+   }
+   return(0);
+}
+
+int smux_snmp_select_list_get_length()
+{
+   return(sdlen);
+}
+
+int smux_snmp_select_list_get_SD_from_List(int pos)
+{
+   if (pos < NUM_SOCKETS)
+   {
+      return(sdlist[pos]);
+   }
+   return(0);
+}
