@@ -224,6 +224,7 @@ disk_free_config(void)
     disks[i].minimumspace = -1;
     disks[i].minpercent = -1;
   }
+  allDisksIncluded = 0;
 }
 
 static void 
@@ -355,6 +356,10 @@ add_device(char *path, char *device, int minspace, int minpercent, int override)
 {
   int index;
 
+  if (!path || !strcmp(path, "none")) {
+    DEBUGMSGTL(("ucd-snmp/disk", "Skipping null path device (%s)\n", device));
+    return;
+  }
   if (numdisks == maxdisks) {
       if (maxdisks == 0) {
           maxdisks = 50;
@@ -389,10 +394,10 @@ add_device(char *path, char *device, int minspace, int minpercent, int override)
   else if(index == -1){
     /* add if and only if the device was found */
     if(device[0] != 0) {
-      copy_nword(path, disks[numdisks].path, 
-		 sizeof(disks[numdisks].path));
-      copy_nword(device, disks[numdisks].device, 
-		 sizeof(disks[numdisks].device));
+      /* The following buffers are cleared above, no need to add '\0' */
+      strncpy(disks[numdisks].path, path, sizeof(disks[numdisks].path) - 1);
+      strncpy(disks[numdisks].device, device,
+              sizeof(disks[numdisks].device) - 1);
       disks[numdisks].minimumspace = minspace;
       disks[numdisks].minpercent   = minpercent;
       numdisks++;  
@@ -417,7 +422,7 @@ int disk_exists(char *path)
 {
   int index;
   for(index = 0; index < numdisks; index++) {
-    DEBUGMSGTL(("ucd-snmp/disk:", "Checking for %s. Found %s at %d ", path, disks[index].path, index));
+    DEBUGMSGTL(("ucd-snmp/disk", "Checking for %s. Found %s at %d\n", path, disks[index].path, index));
     if(strcmp(path, disks[index].path) == 0) {
       return index;
     }
@@ -455,7 +460,12 @@ find_and_add_allDisks(int minpercent)
 #if HAVE_GETMNTENT
 #if HAVE_SETMNTENT
   mntfp = setmntent(ETC_MNTTAB, "r");
-  while (NULL != (mntent = getmntent(mntfp))) {
+  if (!mntfp) {
+      snprintf( tmpbuf, sizeof(tmpbuf), "Can't open %s (setmntent)\n", ETC_MNTTAB );
+      config_perror(tmpbuf);
+      return;
+  }
+  while (mntfp && NULL != (mntent = getmntent(mntfp))) {
     add_device(mntent->mnt_dir, mntent->mnt_fsname, -1, minpercent, 0);
     dummy = 1;
   }
@@ -468,6 +478,11 @@ find_and_add_allDisks(int minpercent)
   }
 #else                           /* getmentent but not setmntent */
   mntfp = fopen(ETC_MNTTAB, "r");
+  if (!mntfp) {
+      snprintf( tmpbuf, sizeof(tmpbuf), "Can't open %s (fopen)\n", ETC_MNTTAB );
+      config_perror(tmpbuf);
+      return;
+  }
   while ((i = getmntent(mntfp, &mnttab)) == 0) {
     add_device(mnttab.mnt_mountp, mnttab.mnt_special, -1, minpercent, 0);
     dummy = 1;
@@ -552,10 +567,15 @@ find_device(char *path)
 #if HAVE_GETMNTENT
 #if HAVE_SETMNTENT
   mntfp = setmntent(ETC_MNTTAB, "r");
-  while (NULL != (mntent = getmntent(mntfp)))
+  if (!mntfp) {
+      snprintf( tmpbuf, sizeof(tmpbuf), "Can't open %s (setmntent)\n", ETC_MNTTAB );
+      config_perror(tmpbuf);
+      return NULL;
+  }
+  while (mntfp && NULL != (mntent = getmntent(mntfp)))
     if (strcmp(path, mntent->mnt_dir) == 0) {
-      copy_nword(mntent->mnt_fsname, device,
-		 sizeof(device));
+      strncpy(device, mntent->mnt_fsname, sizeof(device));
+      device[sizeof(device) - 1] = '\0';
       DEBUGMSGTL(("ucd-snmp/disk", "Disk:  %s\n",
 		  mntent->mnt_fsname));
       break;
@@ -567,6 +587,11 @@ find_device(char *path)
     endmntent(mntfp);
 #else                           /* getmentent but not setmntent */
   mntfp = fopen(ETC_MNTTAB, "r");
+  if (!mntfp) {
+      snprintf( tmpbuf, sizeof(tmpbuf), "Can't open %s (fopen)\n", ETC_MNTTAB );
+      config_perror(tmpbuf);
+      return NULL;
+  }
   while ((i = getmntent(mntfp, &mnttab)) == 0)
     if (strcmp(path, mnttab.mnt_mountp) == 0)
       break;
@@ -576,16 +601,16 @@ find_device(char *path)
     }
   fclose(mntfp);
   if (i == 0) {
-    copy_nword(mnttab.mnt_special, device,
-	       sizeof(device));
+    strncpy(device, mnttab.mnt_special, sizeof(device));
+    device[sizeof(device) - 1] = '\0';
   }
 #endif /* HAVE_SETMNTENT */
 #elif HAVE_FSTAB_H
   stat(path, &stat1);
   setfsent();
   if ((fstab = getfsfile(path))) {
-    copy_nword(fstab->fs_spec, device,
-	       sizeof(device));
+    strncpy(device, fstab->fs_spec, sizeof(device));
+    device[sizeof(device) - 1] = '\0';
   }
   endfsent();
   if (device[0] != '\0') {
@@ -596,7 +621,8 @@ find_device(char *path)
 
 #elif HAVE_STATFS
   if (statfs(path, &statf) == 0) {
-    copy_word(statf.f_mntfromname, device);
+    strncpy(device, statf.f_mntfromname, sizeof(device) - 1);
+    device[sizeof(device) - 1] = '\0';
     DEBUGMSGTL(("ucd-snmp/disk", "Disk:  %s\n",
 		statf.f_mntfromname));
   }
@@ -665,6 +691,7 @@ var_extensible_disk(struct variable *vp,
 #endif
 #endif
 
+tryAgain:
     if (header_simple_table
         (vp, name, length, exact, var_len, write_method, numdisks))
         return (NULL);
@@ -696,6 +723,8 @@ var_extensible_disk(struct variable *vp,
         snmp_log(LOG_ERR, "Couldn't open device %s\n",
                  disks[disknum].device);
         setPerrorstatus("statvfs dev/disk");
+        if (!exact)
+            goto tryAgain;
         return NULL;
     }
 #ifdef STAT_STATFS_FS_DATA
@@ -709,7 +738,9 @@ var_extensible_disk(struct variable *vp,
     vfs.f_bfree = vfs.f_spare[1];
     vfs.f_bavail = vfs.f_spare[2];
 #endif
-    percent = vfs.f_bavail <= 0 ? 100 :
+    percent =
+        vfs.f_blocks == 0 ? 0 :
+        vfs.f_bavail <= 0 ? 100 :
         (int) ((double) (vfs.f_blocks - vfs.f_bfree) /
                (double) (vfs.f_blocks -
                          (vfs.f_bfree - vfs.f_bavail)) * 100.0 + 0.5);
@@ -781,6 +812,8 @@ var_extensible_disk(struct variable *vp,
         snmp_log(LOG_ERR, "Couldn't open device %s\n",
                  disks[disknum].device);
         setPerrorstatus("open dev/disk");
+        if (!exact)
+            goto tryAgain;
         return (NULL);
     }
     lseek(file, (long) (SBLOCK * DEV_BSIZE), 0);
@@ -789,6 +822,8 @@ var_extensible_disk(struct variable *vp,
         snmp_log(LOG_ERR, "Error reading device %s\n",
                  disks[disknum].device);
         close(file);
+        if (!exact)
+            goto tryAgain;
         return (NULL);
     }
     close(file);
@@ -799,8 +834,9 @@ var_extensible_disk(struct variable *vp,
     availblks = totalblks * (100 - filesys.fs_minfree) / 100;
     avail = availblks > used ? availblks - used : 0;
     percent =
-        availblks ==
-        0 ? 100 : (int) ((double) used / (double) totalblks * 100.0 + 0.5);
+        totalblks == 0 ? 0 :
+        availblks == 0 ? 100 :
+        (int) ((double) used / (double) totalblks * 100.0 + 0.5);
     multiplier = (float)filesys.fs_fsize / (float)1024.0;
     iserror =
         (disks[disknum].minimumspace >= 0
