@@ -52,7 +52,7 @@ static kstat_io_t kio;
 static int      cache_disknr = -1;
 #endif                          /* solaris2 */
 
-#if defined(aix4) || defined(aix5)
+#if defined(aix4) || defined(aix5) || defined(aix6)
 /*
  * handle disk statistics via libperfstat
  */
@@ -190,7 +190,7 @@ init_diskio(void)
     IOMasterPort(bootstrap_port, &masterPort);
 #endif
 
-#if defined(aix4) || defined(aix5)
+#if defined(aix4) || defined(aix5) || defined(aix6)
     /*
      * initialize values to gather information on first request
      */
@@ -282,12 +282,12 @@ var_diskio(struct variable * vp,
         long_ret = (uint32_t) kio.nwritten;
         return (u_char *) & long_ret;
     case DISKIO_NREADX:
-        *var_len = 8;
+        *var_len = sizeof(struct counter64);
         c64_ret.low = kio.nread & 0xffffffff;;
         c64_ret.high = kio.nread >> 32;
         return (u_char *) & c64_ret;
     case DISKIO_NWRITTENX:
-        *var_len = 8;
+        *var_len = sizeof(struct counter64);
         c64_ret.low = kio.nwritten & 0xffffffff;;
         c64_ret.high = kio.nwritten >> 32;
         return (u_char *) & c64_ret;
@@ -608,7 +608,7 @@ var_diskio(struct variable * vp,
 #endif
         return (u_char *) & long_ret;
     case DISKIO_NREADX:
-        *var_len = 8;
+        *var_len = sizeof(struct counter64);
 #if defined(freebsd5) && __FreeBSD_version >= 500107
         longlong_ret = stat->dinfo->devices[indx].bytes[DEVSTAT_READ];
 #else
@@ -618,7 +618,7 @@ var_diskio(struct variable * vp,
         c64_ret.high = longlong_ret >> 32;
         return (u_char *) & c64_ret;
     case DISKIO_NWRITTENX:
-        *var_len = 8;
+        *var_len = sizeof(struct counter64);
 #if defined(freebsd5) && __FreeBSD_version >= 500107
         longlong_ret = stat->dinfo->devices[indx].bytes[DEVSTAT_WRITE];
 #else
@@ -811,24 +811,24 @@ var_diskio(struct variable * vp,
       *var_len = strlen(head.indices[indx].name);
       return (u_char *) head.indices[indx].name;
     case DISKIO_NREAD:
-      long_ret = head.indices[indx].rsect*512;
+      long_ret = (head.indices[indx].rsect*512) & 0xffffffff;
       return (u_char *) & long_ret;
     case DISKIO_NWRITTEN:
-      long_ret = head.indices[indx].wsect*512;
+      long_ret = (head.indices[indx].wsect*512) & 0xffffffff;
       return (u_char *) & long_ret;
     case DISKIO_READS:
-      long_ret = head.indices[indx].rio;
+      long_ret = head.indices[indx].rio & 0xffffffff;
       return (u_char *) & long_ret;
     case DISKIO_WRITES:
-      long_ret = head.indices[indx].wio;
+      long_ret = head.indices[indx].wio & 0xffffffff;
       return (u_char *) & long_ret;
     case DISKIO_NREADX:
-      *var_len = 8;
+      *var_len = sizeof(struct counter64);
       c64_ret.low = head.indices[indx].rsect * 512 & 0xffffffff;
       c64_ret.high = head.indices[indx].rsect >> (32 - 9);
       return (u_char *) & c64_ret;
     case DISKIO_NWRITTENX:
-      *var_len = 8;
+      *var_len = sizeof(struct counter64);
       c64_ret.low = head.indices[indx].wsect * 512 & 0xffffffff;
       c64_ret.high = head.indices[indx].wsect >> (32 - 9);
       return (u_char *) & c64_ret;
@@ -1067,7 +1067,7 @@ var_diskio(struct variable * vp,
 #endif                          /* darwin */
 
 
-#if defined(aix4) || defined(aix5)
+#if defined(aix4) || defined(aix5) || defined(aix6)
 /*
  * collect statistics for all disks
  */
@@ -1149,12 +1149,12 @@ var_diskio(struct variable * vp,
         long_ret = (signed long) 0;	/* AIX has just one value for read/write transfers */
         return (u_char *) & long_ret;
     case DISKIO_NREADX:
-        *var_len = 8;
+        *var_len = sizeof(struct counter64);
         c64_ret.low = (ps_disk[indx].rblks * ps_disk[indx].bsize) & 0xffffffff;;
         c64_ret.high = (ps_disk[indx].rblks * ps_disk[indx].bsize) >> 32;
         return (u_char *) & c64_ret;
     case DISKIO_NWRITTENX:
-        *var_len = 8;
+        *var_len = sizeof(struct counter64);
         c64_ret.low = (ps_disk[indx].wblks * ps_disk[indx].bsize) & 0xffffffff;;
         c64_ret.high = (ps_disk[indx].wblks * ps_disk[indx].bsize) >> 32;
         return (u_char *) & c64_ret;
