@@ -399,7 +399,35 @@ sh_count_procs(char *procname)
     return ret;
 }
 
-#elif OSTYPE == NETSNMP_LINUXID
+#elif defined(aix4) || defined(aix5) || defined(aix6)
+#include <procinfo.h>
+#include <sys/types.h>
+
+struct procsinfo pinfo;
+char pinfo_name[256];
+
+int
+sh_count_procs(char *procname)
+{
+    pid_t index;
+    int count;
+    char *sep;
+    
+    index = 0;
+    count = 0;
+
+    while(getprocs(&pinfo, sizeof(pinfo), NULL, 0, &index, 1) == 1) {
+        strncpy(pinfo_name, pinfo.pi_comm, 256);
+        pinfo_name[255] = 0;
+        sep = strchr(pinfo_name, ' ');
+        if(sep != NULL) *sep = 0;
+        if(strcmp(procname, pinfo_name) == 0) count++;
+    }
+
+    return count;
+}
+
+#elif NETSNMP_OSTYPE == NETSNMP_LINUXID
 
 #include <dirent.h>
 #include <fcntl.h>
@@ -410,6 +438,7 @@ sh_count_procs(char *procname)
 {
     DIR *dir;
     char cmdline[512], *tmpc;
+    char state[64];
     struct dirent *ent;
 #ifdef USE_PROC_CMDLINE
     int fd;
@@ -423,7 +452,7 @@ sh_count_procs(char *procname)
 #ifdef USE_PROC_CMDLINE  /* old method */
       /* read /proc/XX/cmdline */
       sprintf(cmdline,"/proc/%s/cmdline",ent->d_name);
-      if((fd = open(cmdline, O_RDONLY)) < 0) break;
+      if((fd = open(cmdline, O_RDONLY)) < 0) continue;
       len = read(fd,cmdline,sizeof(cmdline) - 1);
       close(fd);
       if(len <= 0) continue;
@@ -435,13 +464,21 @@ sh_count_procs(char *procname)
       /* read /proc/XX/status */
       sprintf(cmdline,"/proc/%s/status",ent->d_name);
       if ((status = fopen(cmdline, "r")) == NULL)
-          break;
+          continue;
       if (fgets(cmdline, sizeof(cmdline), status) == NULL) {
           fclose(status);
           break;
       }
+      /* Grab the state of the process as well
+       * (so we can ignore zombie processes)
+       * XXX: Assumes the second line is the status
+       */
+      if (fgets(state, sizeof(state), status) == NULL) {
+          state[0]='\0';
+      }
       fclose(status);
       cmdline[sizeof(cmdline)-1] = '\0';
+      state[sizeof(state)-1] = '\0';
       /* XXX: assumes Name: is first */
       if (strncmp("Name:",cmdline, 5) != 0)
           break;
@@ -456,8 +493,13 @@ sh_count_procs(char *procname)
       DEBUGMSGTL(("proc","Comparing wanted %s against %s\n",
                   procname, tmpc));
       if(len==plen && !strncmp(tmpc,procname,plen)) {
-          total++;
-          DEBUGMSGTL(("proc", " Matched.  total count now=%d\n", total));
+          /* Do not count zombie process as they are not running processes */
+          if ( strstr(state, "zombie") == NULL ) {
+              total++;
+              DEBUGMSGTL(("proc", " Matched.  total count now=%d\n", total));
+          } else {
+              DEBUGMSGTL(("proc", " Skipping zombie process.\n"));
+          }
       }
 #endif      
     }
@@ -465,7 +507,7 @@ sh_count_procs(char *procname)
     return total;
 }
 
-#elif OSTYPE == NETSNMP_ULTRIXID
+#elif NETSNMP_OSTYPE == NETSNMP_ULTRIXID
 
 #define	NPROCS		32      /* number of proces to read at once */
 
@@ -646,7 +688,7 @@ getstruct(off_t loc, char *name, off_t dest, int size)
         return (0);
     return (1);
 }
-#elif OSTYPE == NETSNMP_SOLARISID
+#elif NETSNMP_OSTYPE == NETSNMP_SOLARISID
 
 #ifdef _SLASH_PROC_METHOD_
 
