@@ -224,7 +224,7 @@ int __cdecl     _tmain(int argc, TCHAR * argv[]);
 int             main(int, char **);
 #endif
 
-#if defined(USING_AGENTX_SUBAGENT_MODULE) && !defined(SNMPTRAPD_DISABLE_AGENTX)
+#if defined(USING_AGENTX_SUBAGENT_MODULE) && !defined(NETSNMP_SNMPTRAPD_DISABLE_AGENTX)
 extern void            subagent_init(void);
 #endif
 
@@ -352,7 +352,7 @@ usage(void)
     fprintf(stderr, "  \t\t\t  (followed -quiet to prevent message popups)\n");
 #endif
     fprintf(stderr, "  -v, --version\t\tdisplay version information\n");
-#if defined(USING_AGENTX_SUBAGENT_MODULE) && !defined(SNMPTRAPD_DISABLE_AGENTX)
+#if defined(USING_AGENTX_SUBAGENT_MODULE) && !defined(NETSNMP_SNMPTRAPD_DISABLE_AGENTX)
     fprintf(stderr, "  -x ADDRESS\t\tuse ADDRESS as AgentX address\n");
 #endif
     fprintf(stderr,
@@ -666,7 +666,7 @@ main(int argc, char *argv[])
     fd_set          readfds,writefds,exceptfds;
     struct timeval  timeout, *tvp;
     char           *cp, *listen_ports = NULL;
-#if defined(USING_AGENTX_SUBAGENT_MODULE) && !defined(SNMPTRAPD_DISABLE_AGENTX)
+#if defined(USING_AGENTX_SUBAGENT_MODULE) && !defined(NETSNMP_SNMPTRAPD_DISABLE_AGENTX)
     int             agentx_subagent = 1;
 #endif
     netsnmp_trapd_handler *traph;
@@ -674,11 +674,21 @@ main(int argc, char *argv[])
     extern void init_perl(void);
 #endif
 
+#ifndef WIN32
+    /*
+     * close all non-standard file descriptors we may have
+     * inherited from the shell.
+     */
+    for (i = getdtablesize() - 1; i > 2; --i) {
+        (void) close(i);
+    }
+#endif /* #WIN32 */
+    
 #ifdef SIGTERM
     signal(SIGTERM, term_handler);
 #endif
 #ifdef SIGHUP
-    signal(SIGHUP, hup_handler);
+    signal(SIGHUP, SIG_IGN);   /* do not terminate on early SIGHUP */
 #endif
 #ifdef SIGINT
     signal(SIGINT, term_handler);
@@ -687,6 +697,10 @@ main(int argc, char *argv[])
     signal(SIGPIPE, SIG_IGN);   /* 'Inline' failure of wayward readers */
 #endif
 
+#ifdef SIGHUP
+    signal(SIGHUP, SIG_IGN);   /* do not terminate on early SIGHUP */
+#endif
+
     /*
      * register our configuration handlers now so -H properly displays them 
      */
@@ -1071,6 +1085,7 @@ main(int argc, char *argv[])
         traph = netsnmp_add_global_traphandler(NETSNMPTRAPD_PRE_HANDLER,
                                                syslog_handler);
         traph->authtypes = TRAP_AUTH_LOG;
+        snmp_enable_syslog();
     } else {
         traph = netsnmp_add_global_traphandler(NETSNMPTRAPD_PRE_HANDLER,
                                                print_handler);
@@ -1095,7 +1110,7 @@ main(int argc, char *argv[])
 	 */
     }
 
-#if defined(USING_AGENTX_SUBAGENT_MODULE) && !defined(SNMPTRAPD_DISABLE_AGENTX)
+#if defined(USING_AGENTX_SUBAGENT_MODULE) && !defined(NETSNMP_SNMPTRAPD_DISABLE_AGENTX)
     /*
      * we're an agentx subagent? 
      */
@@ -1124,7 +1139,7 @@ main(int argc, char *argv[])
      */
     init_agent("snmptrapd");
 
-#if defined(USING_AGENTX_SUBAGENT_MODULE) && !defined(SNMPTRAPD_DISABLE_AGENTX)
+#if defined(USING_AGENTX_SUBAGENT_MODULE) && !defined(NETSNMP_SNMPTRAPD_DISABLE_AGENTX)
     /*
      * initialize local modules 
      */
@@ -1154,12 +1169,12 @@ main(int argc, char *argv[])
         register_snmpEngine_scalars_context("snmptrapd");
 #endif
     }
-#endif /* USING_AGENTX_SUBAGENT_MODULE && !SNMPTRAPD_DISABLE_AGENTX */
+#endif /* USING_AGENTX_SUBAGENT_MODULE && !NETSNMP_SNMPTRAPD_DISABLE_AGENTX */
 
     /* register our authorization handler */
     init_netsnmp_trapd_auth();
 
-#if defined(USING_AGENTX_SUBAGENT_MODULE) && !defined(SNMPTRAPD_DISABLE_AGENTX)
+#if defined(USING_AGENTX_SUBAGENT_MODULE) && !defined(NETSNMP_SNMPTRAPD_DISABLE_AGENTX)
     if (agentx_subagent) {
 #ifdef USING_AGENT_NSVACMACCESSTABLE_MODULE
         extern void init_register_nsVacm_context(const char *);
@@ -1194,6 +1209,10 @@ main(int argc, char *argv[])
      */
     init_snmp("snmptrapd");
 
+#ifdef SIGHUP
+    signal(SIGHUP, hup_handler);
+#endif
+
     if (trap1_fmt_str_remember) {
         free_trap1_fmt();
         free_trap2_fmt();
@@ -1283,26 +1302,15 @@ main(int argc, char *argv[])
 
     while (cp != NULL) {
         char *sep = strchr(cp, ',');
-        char  listen_name[128];
-        char *cp2 = strchr(cp, ':');
 
         if (sep != NULL) {
             *sep = 0;
         }
 
-           /*
-            * Make sure this defaults to listening on port 162
-            */
-        if (!cp2) {
-            snprintf(listen_name, sizeof(listen_name), "%s:162", cp);
-            cp2 = listen_name;
-        } else {
-            cp2 = cp;
-        }
-        transport = netsnmp_transport_open_server("snmptrap", cp2);
+        transport = netsnmp_transport_open_server("snmptrap", cp);
         if (transport == NULL) {
             snmp_log(LOG_ERR, "couldn't open %s -- errno %d (\"%s\")\n",
-                     cp2, errno, strerror(errno));
+                     cp, errno, strerror(errno));
             snmptrapd_close_sessions(sess_list);
             SOCK_CLEANUP;
             exit(1);
@@ -1391,7 +1399,9 @@ main(int argc, char *argv[])
             trapd_update_config();
             if (trap1_fmt_str_remember) {
                 free_trap1_fmt();
+                free_trap2_fmt();
                 print_format1 = strdup(trap1_fmt_str_remember);
+                print_format2 = strdup(trap1_fmt_str_remember);
             }
             reconfig = 0;
         }
