@@ -132,9 +132,12 @@ parse_forward(const char *token, char *line)
     char            buf[STRINGMAX];
     oid             obuf[MAX_OID_LEN];
     size_t          olen = MAX_OID_LEN;
-    char           *cptr;
+    char           *cptr, *cp;
     netsnmp_trapd_handler *traph;
+    int             flags = 0;
 
+    memset( buf, 0, sizeof(buf));
+    memset(obuf, 0, sizeof(obuf));
     cptr = copy_nword(line, buf, sizeof(buf));
     DEBUGMSGTL(("read_config:forward", "registering forward for: "));
     if (!strcmp(buf, "default")) {
@@ -144,6 +147,18 @@ parse_forward(const char *token, char *line)
         else
             traph = netsnmp_add_default_traphandler( forward_handler );
     } else {
+        cp = buf+strlen(buf)-1;
+        if ( *cp == '*' ) {
+            flags |= NETSNMP_TRAPHANDLER_FLAG_MATCH_TREE;
+            *(cp--) = '\0';
+            if ( *cp == '.' ) {
+                /* 
+                 * Distinguish between 'oid.*' & 'oid*'
+                 */
+                flags |= NETSNMP_TRAPHANDLER_FLAG_STRICT_SUBTREE;
+                *(cp--) = '\0';
+            }
+        }
 
         if (!read_objid(buf, obuf, &olen)) {
             char            buf1[STRINGMAX];
@@ -163,6 +178,7 @@ parse_forward(const char *token, char *line)
     DEBUGMSG(("read_config:forward", "\n"));
 
     if (traph) {
+        traph->flags = flags;
         traph->authtypes = TRAP_AUTH_NET;
         traph->token = strdup(cptr);
     }
@@ -190,28 +206,35 @@ parse_format(const char *token, char *line)
         return;
     }
 
-    *cp = '\0';
-    cp++;
+    *(cp++) = '\0';
+    while (*cp && !isspace(*cp))
+        cp++;
 
     /*
      * OK - now "line" contains the format type,
      *      and cp points to the actual format string.
      * So update the appropriate pointer(s).
-     *
-     * XXX - the previous values really need to be freed too
      */
-    if (!strcmp( line, "print1"))
+    if (!strcmp( line, "print1")) {
+        SNMP_FREE( print_format1 );
         print_format1 = strdup(cp);
-    else if (!strcmp( line, "print2"))
+    } else if (!strcmp( line, "print2")) {
+        SNMP_FREE( print_format2 );
         print_format2 = strdup(cp);
-    else if (!strcmp( line, "print")) {
+    } else if (!strcmp( line, "print")) {
+        SNMP_FREE( print_format1 );
+        SNMP_FREE( print_format2 );
         print_format1 = strdup(cp);
         print_format2 = strdup(cp);
-    } else if (!strcmp( line, "syslog1"))
+    } else if (!strcmp( line, "syslog1")) {
+        SNMP_FREE( syslog_format1 );
         syslog_format1 = strdup(cp);
-    else if (!strcmp( line, "syslog2"))
+    } else if (!strcmp( line, "syslog2")) {
+        SNMP_FREE( syslog_format2 );
         syslog_format2 = strdup(cp);
-    else if (!strcmp( line, "syslog")) {
+    } else if (!strcmp( line, "syslog")) {
+        SNMP_FREE( syslog_format1 );
+        SNMP_FREE( syslog_format2 );
         syslog_format1 = strdup(cp);
         syslog_format2 = strdup(cp);
     }
@@ -996,7 +1019,7 @@ int   forward_handler( netsnmp_pdu           *pdu,
     return NETSNMPTRAPD_HANDLER_OK;
 }
 
-#if defined(USING_NOTIFICATION_LOG_MIB_NOTIFICATION_LOG_MODULE) && defined(USING_AGENTX_SUBAGENT_MODULE) && !defined(SNMPTRAPD_DISABLE_AGENTX)
+#if defined(USING_NOTIFICATION_LOG_MIB_NOTIFICATION_LOG_MODULE) && defined(USING_AGENTX_SUBAGENT_MODULE) && !defined(NETSNMP_SNMPTRAPD_DISABLE_AGENTX)
 /*
  *  "Notification" handler for implementing NOTIFICATION-MIB
  *  		(presumably)
