@@ -426,6 +426,10 @@ main(int argc, char *argv[])
      * create PDU for SET request and add object names and values to request 
      */
     pdu = snmp_pdu_create(SNMP_MSG_SET);
+    if (!pdu) {
+        fprintf(stderr, "Failed to create request\n");
+        exit(1);
+    }
 
 
     if (strcmp(argv[arg], CMD_PASSWD_NAME) == 0) {
@@ -868,6 +872,10 @@ main(int argc, char *argv[])
 
         /* fetch the needed diffie helman parameters */
         dhpdu = snmp_pdu_create(SNMP_MSG_GET);
+        if (!dhpdu) {
+            fprintf(stderr, "Failed to create DH request\n");
+            exit(1);
+        }
 
         /* get the current DH parameters */
         snmp_add_null_var(dhpdu, usmDHParameters, usmDHParameters_len);
