@@ -13,6 +13,9 @@ netsnmp_register_default_domain(const char* application, const char* domain);
 extern const char*
 netsnmp_lookup_default_domain(const char* application);
 
+extern const char* const *
+netsnmp_lookup_default_domains(const char* application);
+
 extern void
 netsnmp_clear_default_domain(void);
 
@@ -26,6 +29,9 @@ netsnmp_lookup_default_target(const char* application, const char* domain);
 extern void
 netsnmp_clear_default_target(void);
 
+extern void
+netsnmp_register_service_handlers(void);
+
 #ifdef __cplusplus
 }
 #endif
