@@ -139,72 +139,18 @@ static int      config_errors;
 
 struct config_files *config_files = NULL;
 
-struct config_line *
-register_prenetsnmp_mib_handler(const char *type,
-                                const char *token,
-                                void (*parser) (const char *, char *),
-                                void (*releaser) (void), const char *help)
-{
-    struct config_line *ltmp;
-    ltmp = register_config_handler(type, token, parser, releaser, help);
-    if (ltmp != NULL)
-        ltmp->config_time = PREMIB_CONFIG;
-    return (ltmp);
-}
-
-struct config_line *
-register_app_prenetsnmp_mib_handler(const char *token,
-                                    void (*parser) (const char *, char *),
-                                    void (*releaser) (void),
-                                    const char *help)
-{
-    return (register_prenetsnmp_mib_handler
-            (NULL, token, parser, releaser, help));
-}
-
-/**
- * register_config_handler registers handlers for certain tokens specified in
- * certain types of files.
- *
- * Allows a module writer use/register multiple configuration files based off
- * of the type parameter.  A module writer may want to set up multiple
- * configuration files to separate out related tasks/variables or just for
- * management of where to put tokens as the module or modules get more complex
- * in regard to handling token registrations.
- *
- * @param type_param the configuration file used, e.g., if snmp.conf is the file
- *                 where the token is located use "snmp" here.
- *                 If NULL the configuration file used will be snmpd.conf.
- *
- * @param token    the token being parsed from the file.  Must be non-NULL.
- *
- * @param parser   the handler function pointer that use  the specified
- *                 token and the rest of the line to do whatever is required
- *                 Should be non-NULL in order to make use of this API.
- *
- * @param releaser if non-NULL, the function specified is called if
- *                 and when the configuration files are re-read.  This function
- *                 should free any resources allocated by the token handler
- *                 function.
- *
- * @param help     if non-NULL, used to display help information on the expected 
- *	           arguments after the token.
- *      
- * @return Pointer to a new config line entry or NULL on error.
- */
-struct config_line *
-register_config_handler(const char *type_param,
-                        const char *token,
-                        void (*parser) (const char *, char *),
-                        void (*releaser) (void), const char *help)
+static struct config_line *
+internal_register_config_handler(const char *type_param,
+				 const char *token,
+				 void (*parser) (const char *, char *),
+				 void (*releaser) (void), const char *help,
+				 int when)
 {
     struct config_files **ctmp = &config_files;
-    struct config_line **ltmp, *ltmp2;
-    const char     *type = type_param;
-    char           *cptr, buf[STRINGMAX];
-    char           *st;
+    struct config_line  **ltmp;
+    const char           *type = type_param;
 
-    if (type == NULL) {
+    if (type == NULL || *type == '\0') {
         type = netsnmp_ds_get_string(NETSNMP_DS_LIBRARY_ID, 
 				     NETSNMP_DS_LIB_APPTYPE);
     }
@@ -212,15 +158,21 @@ register_config_handler(const char *type_param,
     /*
      * Handle multiple types (recursively)
      */
-    cptr = strchr( type, ':' );
-    if (cptr) {
-        strncpy(buf, type, STRINGMAX);
+    if (strchr(type, ':')) {
+        struct config_line *ltmp2 = NULL;
+        char                buf[STRINGMAX];
+        char               *cptr = buf;
+        strncpy(buf, type, STRINGMAX - 1);
         buf[STRINGMAX - 1] = '\0';
-        ltmp2 = NULL;
-        cptr = strtok_r(buf, ":", &st);
         while (cptr) {
-            ltmp2 = register_config_handler(cptr, token, parser, releaser, help);
-            cptr  = strtok_r(NULL, ":", &st);
+            char* c = cptr;
+            cptr = strchr(cptr, ':');
+            if(cptr) {
+                *cptr = '\0';
+                ++cptr;
+            }
+            ltmp2 = internal_register_config_handler(c, token, parser,
+                                                     releaser, help, when);
         }
         return ltmp2;
     }
@@ -259,7 +211,7 @@ register_config_handler(const char *type_param,
             return NULL;
         }
 
-        (*ltmp)->config_time = NORMAL_CONFIG;
+        (*ltmp)->config_time = when;
         (*ltmp)->config_token = strdup(token);
         if (help != NULL)
             (*ltmp)->help = strdup(help);
@@ -276,6 +228,68 @@ register_config_handler(const char *type_param,
 
 }                               /* end register_config_handler() */
 
+struct config_line *
+register_prenetsnmp_mib_handler(const char *type,
+                                const char *token,
+                                void (*parser) (const char *, char *),
+                                void (*releaser) (void), const char *help)
+{
+    return internal_register_config_handler(type, token, parser, releaser,
+					    help, PREMIB_CONFIG);
+}
+
+struct config_line *
+register_app_prenetsnmp_mib_handler(const char *token,
+                                    void (*parser) (const char *, char *),
+                                    void (*releaser) (void),
+                                    const char *help)
+{
+    return (register_prenetsnmp_mib_handler
+            (NULL, token, parser, releaser, help));
+}
+
+/**
+ * register_config_handler registers handlers for certain tokens specified in
+ * certain types of files.
+ *
+ * Allows a module writer use/register multiple configuration files based off
+ * of the type parameter.  A module writer may want to set up multiple
+ * configuration files to separate out related tasks/variables or just for
+ * management of where to put tokens as the module or modules get more complex
+ * in regard to handling token registrations.
+ *
+ * @param type     the configuration file used, e.g., if snmp.conf is the
+ *                 file where the token is located use "snmp" here.
+ *                 Multiple colon separated tokens might be used.
+ *                 If NULL or "" then the configuration file used will be
+ *                 \<application\>.conf.
+ *
+ * @param token    the token being parsed from the file.  Must be non-NULL.
+ *
+ * @param parser   the handler function pointer that use  the specified
+ *                 token and the rest of the line to do whatever is required
+ *                 Should be non-NULL in order to make use of this API.
+ *
+ * @param releaser if non-NULL, the function specified is called if
+ *                 and when the configuration files are re-read.  This function
+ *                 should free any resources allocated by the token handler
+ *                 function.
+ *
+ * @param help     if non-NULL, used to display help information on the
+ *                 expected arguments after the token.
+ *
+ * @return Pointer to a new config line entry or NULL on error.
+ */
+struct config_line *
+register_config_handler(const char *type,
+			const char *token,
+			void (*parser) (const char *, char *),
+			void (*releaser) (void), const char *help)
+{
+    return internal_register_config_handler(type, token, parser, releaser,
+					    help, NORMAL_CONFIG);
+}
+
 struct config_line *
 register_app_config_handler(const char *token,
                             void (*parser) (const char *, char *),
@@ -301,12 +315,10 @@ void
 unregister_config_handler(const char *type_param, const char *token)
 {
     struct config_files **ctmp = &config_files;
-    struct config_line **ltmp, *ltmp2;
-    const char     *type = type_param;
-    char           *cptr, buf[STRINGMAX];
-    char           *st;
+    struct config_line  **ltmp;
+    const char           *type = type_param;
 
-    if (type == NULL) {
+    if (type == NULL || *type == '\0') {
         type = netsnmp_ds_get_string(NETSNMP_DS_LIBRARY_ID, 
 				     NETSNMP_DS_LIB_APPTYPE);
     }
@@ -314,14 +326,19 @@ unregister_config_handler(const char *type_param, const char *token)
     /*
      * Handle multiple types (recursively)
      */
-    cptr = strchr( type, ':' );
-    if (cptr) {
-        strncpy(buf, type, STRINGMAX);
+    if (strchr(type, ':')) {
+        char                buf[STRINGMAX];
+        char               *cptr = buf;
+        strncpy(buf, type, STRINGMAX - 1);
         buf[STRINGMAX - 1] = '\0';
-        cptr = strtok_r(buf, ":", &st);
         while (cptr) {
-            unregister_config_handler(cptr, token);
-            cptr  = strtok_r(NULL, ":", &st);
+            char* c = cptr;
+            cptr = strchr(cptr, ':');
+            if(cptr) {
+                *cptr = '\0';
+                ++cptr;
+            }
+            unregister_config_handler(c, token);
         }
         return;
     }
@@ -351,7 +368,7 @@ unregister_config_handler(const char *type_param, const char *token)
         /*
          * found it at the top of the list 
          */
-        ltmp2 = (*ltmp)->next;
+        struct config_line *ltmp2 = (*ltmp)->next;
         SNMP_FREE((*ltmp)->config_token);
         SNMP_FREE((*ltmp)->help);
         SNMP_FREE(*ltmp);
@@ -363,9 +380,9 @@ unregister_config_handler(const char *type_param, const char *token)
         ltmp = &((*ltmp)->next);
     }
     if ((*ltmp)->next != NULL) {
+        struct config_line *ltmp2 = (*ltmp)->next->next;
         SNMP_FREE((*ltmp)->next->config_token);
         SNMP_FREE((*ltmp)->next->help);
-        ltmp2 = (*ltmp)->next->next;
         SNMP_FREE((*ltmp)->next);
         (*ltmp)->next = ltmp2;
     }
@@ -967,7 +984,7 @@ get_persistent_directory()
 {
     if (NULL == netsnmp_ds_get_string(NETSNMP_DS_LIBRARY_ID, 
 				      NETSNMP_DS_LIB_PERSISTENT_DIR)) {
-        char *persdir = netsnmp_getenv("SNMP_PERSISTENT_DIR");
+        const char *persdir = netsnmp_getenv("SNMP_PERSISTENT_DIR");
         if (NULL == persdir)
             persdir = NETSNMP_PERSISTENT_DIRECTORY;
         set_persistent_directory(persdir);
@@ -1034,13 +1051,34 @@ read_config_files_in_path(const char *path, struct config_files *ctmp,
                 ctmp->fileHeader, envconfpath, perspath));
     cptr1 = cptr2 = envconfpath;
     done = 0;
-    while (*cptr2 != 0) {
+    while ((!done) && (*cptr2 != 0)) {
         while (*cptr1 != 0 && *cptr1 != ENV_SEPARATOR_CHAR)
             cptr1++;
         if (*cptr1 == 0)
             done = 1;
         else
             *cptr1 = 0;
+
+        DEBUGMSGTL(("read_config", " config dir: %s\n", cptr2 ));
+        if (stat(cptr2, &statbuf) != 0) {
+            /*
+             * Directory not there, continue 
+             */
+            DEBUGMSGTL(("read_config", " Directory not present: %s\n", cptr2 ));
+            cptr2 = ++cptr1;
+            continue;
+        }
+#ifdef S_ISDIR
+        if (!S_ISDIR(statbuf.st_mode)) {
+            /*
+             * Not a directory, continue 
+             */
+            DEBUGMSGTL(("read_config", " Not a directory: %s\n", cptr2 ));
+            cptr2 = ++cptr1;
+            continue;
+        }
+#endif
+
         /*
          * for proper persistent storage retrival, we need to read old backup
          * copies of the previous storage files.  If the application in
