@@ -39,6 +39,9 @@
 #if HAVE_NETDB_H
 #include <netdb.h>
 #endif
+#if HAVE_SYS_UIO_H
+#include <sys/uio.h>
+#endif
 
 #if HAVE_WINSOCK_H
 #include <winsock2.h>
@@ -101,11 +104,13 @@ netsnmp_udp_fmtaddr(netsnmp_transport *t, void *data, int len)
 	char tmp[64];
         to = (struct sockaddr_in *) &(addr_pair->remote_addr);
         if (to == NULL) {
-            return strdup("UDP: unknown");
+            sprintf(tmp, "UDP: [%s]->unknown",
+                    inet_ntoa(addr_pair->local_addr));
+        } else {
+            sprintf(tmp, "UDP: [%s]->", inet_ntoa(addr_pair->local_addr));
+            sprintf(tmp + strlen(tmp), "[%s]:%hd",
+                    inet_ntoa(to->sin_addr), ntohs(to->sin_port));
         }
-
-        sprintf(tmp, "UDP: [%s]:%hu",
-                inet_ntoa(to->sin_addr), ntohs(to->sin_port));
         return strdup(tmp);
     }
 }
@@ -116,7 +121,7 @@ netsnmp_udp_fmtaddr(netsnmp_transport *t, void *data, int len)
 
 # define netsnmp_dstaddr(x) (&(((struct in_pktinfo *)(CMSG_DATA(x)))->ipi_addr))
 
-static int netsnmp_udp_recvfrom(int s, char *buf, int len, struct sockaddr *from, socklen_t *fromlen, struct in_addr *dstip)
+static int netsnmp_udp_recvfrom(int s, void *buf, int len, struct sockaddr *from, socklen_t *fromlen, struct in_addr *dstip)
 {
     int r;
     struct iovec iov[1];
@@ -153,32 +158,29 @@ static int netsnmp_udp_recvfrom(int s, char *buf, int len, struct sockaddr *from
 }
 
 static int netsnmp_udp_sendto(int fd, struct in_addr *srcip, struct sockaddr *remote,
-			char *data, int len)
+			void *data, int len)
 {
     struct iovec iov = { data, len };
     struct {
         struct cmsghdr cm;
         struct in_pktinfo ipi;
-    } cmsg = {
-        .cm = {
-            .cmsg_len	= sizeof(struct cmsghdr) + sizeof(struct in_pktinfo),
-            .cmsg_level	= SOL_IP,
-            .cmsg_type	= IP_PKTINFO,
-        },
-        .ipi = {
-            .ipi_ifindex	= 0,
-            .ipi_spec_dst	= srcip ? srcip->s_addr : 0,
-        },
-    };
-    struct msghdr m = {
-        .msg_name	= remote,
-        .msg_namelen	= sizeof(struct sockaddr_in),
-        .msg_iov	= &iov,
-        .msg_iovlen	= 1,
-        .msg_control	= &cmsg,
-        .msg_controllen	= sizeof(cmsg),
-        .msg_flags	= 0,
-    };
+    } cmsg;
+    struct msghdr m;
+
+    cmsg.cm.cmsg_len = sizeof(struct cmsghdr) + sizeof(struct in_pktinfo);
+    cmsg.cm.cmsg_level = SOL_IP;
+    cmsg.cm.cmsg_type = IP_PKTINFO;
+    cmsg.ipi.ipi_ifindex = 0;
+    cmsg.ipi.ipi_spec_dst.s_addr = (srcip ? srcip->s_addr : INADDR_ANY);
+
+    m.msg_name		= remote;
+    m.msg_namelen	= sizeof(struct sockaddr_in);
+    m.msg_iov		= &iov;
+    m.msg_iovlen	= 1;
+    m.msg_control	= &cmsg;
+    m.msg_controllen	= sizeof(cmsg);
+    m.msg_flags		= 0;
+
     return sendmsg(fd, &m, MSG_NOSIGNAL|MSG_DONTWAIT);
 }
 #endif /* linux && IP_PKTINFO */
@@ -586,28 +588,24 @@ netsnmp_udp_transport(struct sockaddr_in *addr, int local)
     int             rc = 0;
     char           *str = NULL;
     char           *client_socket = NULL;
-    netsnmp_udp_addr_pair *addr_pair = NULL;
+    netsnmp_udp_addr_pair addr_pair;
 
     if (addr == NULL || addr->sin_family != AF_INET) {
         return NULL;
     }
 
-    addr_pair = (netsnmp_udp_addr_pair *) malloc(sizeof(netsnmp_udp_addr_pair));
-    if (addr_pair == NULL) {
-        return NULL;
-    }
-    memset(addr_pair, 0, sizeof(netsnmp_udp_addr_pair));
-    memcpy(&(addr_pair->remote_addr), addr, sizeof(struct sockaddr_in));
+    memset(&addr_pair, 0, sizeof(netsnmp_udp_addr_pair));
+    memcpy(&(addr_pair.remote_addr), addr, sizeof(struct sockaddr_in));
 
     t = (netsnmp_transport *) malloc(sizeof(netsnmp_transport));
     if (t == NULL) {
         return NULL;
     }
 
-    str = netsnmp_udp_fmtaddr(NULL, (void *)addr_pair, 
+    str = netsnmp_udp_fmtaddr(NULL, (void *)&addr_pair,
                                  sizeof(netsnmp_udp_addr_pair));
-    DEBUGMSGTL(("netsnmp_udp", "open %s %s:%d\n", local ? "local" : "remote",
-                str,addr->sin_port));
+    DEBUGMSGTL(("netsnmp_udp", "open %s %s\n", local ? "local" : "remote",
+                str));
     free(str);
 
     memset(t, 0, sizeof(netsnmp_transport));
@@ -646,6 +644,7 @@ netsnmp_udp_transport(struct sockaddr_in *addr, int local)
             if (setsockopt(t->sock, SOL_IP, IP_PKTINFO, &sockopt, sizeof sockopt) == -1) {
                 DEBUGMSGTL(("netsnmp_udp", "couldn't set IP_PKTINFO: %s\n",
                     strerror(errno)));
+                netsnmp_transport_free(t);
                 return NULL;
             }
             DEBUGMSGTL(("netsnmp_udp", "set IP_PKTINFO\n"));
@@ -671,10 +670,24 @@ netsnmp_udp_transport(struct sockaddr_in *addr, int local)
         if (client_socket) {
             struct sockaddr_in client_addr;
             netsnmp_sockaddr_in2(&client_addr, client_socket, NULL);
+            addr_pair.local_addr = client_addr.sin_addr;
             client_addr.sin_port = 0;
-            bind(t->sock, (struct sockaddr *)&client_addr,
+            rc = bind(t->sock, (struct sockaddr *)&client_addr,
                   sizeof(struct sockaddr));
+            if ( rc != 0 ) {
+                DEBUGMSGTL(("netsnmp_udp", "failed to bind for clientaddr: %d %s\n",
+                            errno, strerror(errno)));
+                netsnmp_udp_close(t);
+                netsnmp_transport_free(t);
+                return NULL;
+            }
         }
+
+        str = netsnmp_udp_fmtaddr(NULL, (void *)&addr_pair,
+                 sizeof(netsnmp_udp_addr_pair));
+        DEBUGMSGTL(("netsnmp_udp", "client open %s\n", str));
+        free(str);
+
         /*
          * Save the (remote) address in the
          * transport-specific data pointer for later use by netsnmp_udp_send.
@@ -690,7 +703,7 @@ netsnmp_udp_transport(struct sockaddr_in *addr, int local)
         t->remote[4] = (htons(addr->sin_port) & 0xff00) >> 8;
         t->remote[5] = (htons(addr->sin_port) & 0x00ff) >> 0;
         t->remote_length = 6;
-        memcpy(t->data, addr_pair, sizeof(netsnmp_udp_addr_pair));
+        memcpy(t->data, &addr_pair, sizeof(netsnmp_udp_addr_pair));
         t->data_length = sizeof(netsnmp_udp_addr_pair);
     }
 
@@ -781,18 +794,19 @@ netsnmp_sockaddr_in2(struct sockaddr_in *addr,
                 addr, inpeername ? inpeername : "[NIL]",
                 default_target ? default_target : "[NIL]"));
 
-    if (default_target == NULL ||
-	netsnmp_sockaddr_in2(addr, default_target, NULL) == 0) {
-	int port;
+    memset(addr, 0, sizeof(struct sockaddr_in));
+    addr->sin_addr.s_addr = htonl(INADDR_ANY);
+    addr->sin_family = AF_INET;
+    addr->sin_port = htons((u_short)SNMP_PORT);
 
-	memset(addr, 0, sizeof(struct sockaddr_in));
-	addr->sin_addr.s_addr = htonl(INADDR_ANY);
-	addr->sin_family = AF_INET;
-	port = netsnmp_ds_get_int(NETSNMP_DS_LIBRARY_ID,
-				  NETSNMP_DS_LIB_DEFAULT_PORT);
-	if (port == 0)
-	    port = SNMP_PORT;
-	addr->sin_port = htons((u_short)port);
+    {
+	int port = netsnmp_ds_get_int(NETSNMP_DS_LIBRARY_ID,
+				      NETSNMP_DS_LIB_DEFAULT_PORT);
+
+	if (port != 0) {
+	    addr->sin_port = htons((u_short)port);
+	} else if (default_target != NULL)
+	    netsnmp_sockaddr_in2(addr, default_target, NULL);
     }
 
     if (inpeername != NULL && *inpeername != '\0') {
@@ -873,8 +887,13 @@ netsnmp_sockaddr_in2(struct sockaddr_in *addr,
 
             err = getaddrinfo(peername, NULL, &hint, &addrs);
             if (err != 0) {
+#if HAVE_GAI_STRERROR
                 snmp_log(LOG_ERR, "getaddrinfo: %s %s\n", peername,
                          gai_strerror(err));
+#else
+                snmp_log(LOG_ERR, "getaddrinfo: %s (error %d)\n", peername,
+                         err);
+#endif
                 free(peername);
                 return 0;
             }
@@ -960,7 +979,7 @@ netsnmp_sockaddr_in(struct sockaddr_in *addr,
 #define EXAMPLE_COMMUNITY	"COMMUNITY"
 
 typedef struct _com2SecEntry {
-    char            community[VACMSTRINGLEN];
+    char            community[COMMUNITY_MAX_LEN];
     unsigned long   network;
     unsigned long   mask;
     char            secName[VACMSTRINGLEN];
@@ -975,7 +994,7 @@ netsnmp_udp_parse_security(const char *token, char *param)
 {
     char            secName[VACMSTRINGLEN];
     char            contextName[VACMSTRINGLEN];
-    char            community[VACMSTRINGLEN];
+    char            community[COMMUNITY_MAX_LEN];
     char            source[SNMP_MAXBUF_SMALL];
     char           *cp = NULL;
     const char     *strmask = NULL;
@@ -1023,7 +1042,7 @@ netsnmp_udp_parse_security(const char *token, char *param)
             == 0) {
         config_perror("example config COMMUNITY not properly configured");
         return;
-    } else if (strlen(community) > (VACMSTRINGLEN - 1)) {
+    } else if (strlen(community) > (COMMUNITY_MAX_LEN - 1)) {
         config_perror("community name too long");
         return;
     }
