@@ -110,7 +110,7 @@ netsnmp_udp6_fmtaddr(netsnmp_transport *t, void *data, int len)
         return strdup("UDP/IPv6: unknown");
     } else {
         char addr[INET6_ADDRSTRLEN];
-        char tmp[INET6_ADDRSTRLEN + 8];
+        char tmp[INET6_ADDRSTRLEN + 18];
 
         sprintf(tmp, "UDP/IPv6: [%s]:%hu",
                 inet_ntop(AF_INET6, (void *) &(to->sin6_addr), addr,
@@ -287,7 +287,7 @@ netsnmp_udp6_transport(struct sockaddr_in6 *addr, int local)
             netsnmp_transport_free(t);
             return NULL;
         }
-        t->local = malloc(18);
+        t->local = (unsigned char*)malloc(18);
         if (t->local == NULL) {
             netsnmp_udp6_close(t);
             netsnmp_transport_free(t);
@@ -313,7 +313,7 @@ netsnmp_udp6_transport(struct sockaddr_in6 *addr, int local)
         }
         memcpy(t->data, addr, sizeof(struct sockaddr_in6));
         t->data_length = sizeof(struct sockaddr_in6);
-        t->remote = malloc(18);
+        t->remote = (unsigned char*)malloc(18);
         if (t->remote == NULL) {
             netsnmp_udp6_close(t);
             netsnmp_transport_free(t);
@@ -370,19 +370,18 @@ netsnmp_sockaddr_in6_2(struct sockaddr_in6 *addr,
                 addr, inpeername ? inpeername : "[NIL]",
 		default_target ? default_target : "[NIL]"));
 
-    if (default_target == NULL ||
-	netsnmp_sockaddr_in6_2(addr, default_target, NULL) == 0) {
-	int port;
-	memset(addr, 0, sizeof(struct sockaddr_in6));
-	addr->sin6_family = AF_INET6;
-	addr->sin6_addr = in6addr_any;
-
-	port = netsnmp_ds_get_int(NETSNMP_DS_LIBRARY_ID,
-				  NETSNMP_DS_LIB_DEFAULT_PORT);
-	if (port <= 0)
-	    port = SNMP_PORT;
+    memset(addr, 0, sizeof(struct sockaddr_in6));
+    addr->sin6_family = AF_INET6;
+    addr->sin6_addr = in6addr_any;
+    addr->sin6_port = htons((u_short)SNMP_PORT);
 
+    {
+      int port = netsnmp_ds_get_int(NETSNMP_DS_LIBRARY_ID,
+				    NETSNMP_DS_LIB_DEFAULT_PORT);
+      if (port != 0)
         addr->sin6_port = htons((u_short)port);
+      else if (default_target != NULL)
+	netsnmp_sockaddr_in6_2(addr, default_target, NULL);
     }
 
     if (inpeername != NULL) {
@@ -441,7 +440,7 @@ netsnmp_sockaddr_in6_2(struct sockaddr_in6 *addr,
                                     "IPv6 address with port suffix :%d\n",
                                     atoi(cp + 2)));
                         addr->sin6_port = htons(atoi(cp + 2));
-#ifdef HAVE_IF_NAMETOINDEX
+#if defined(HAVE_IF_NAMETOINDEX) && defined(HAVE_STRUCT_SOCKADDR_IN6_SIN6_SCOPE_ID)
                         addr->sin6_scope_id = if_index;
 #endif
                         goto resolved;
@@ -457,7 +456,7 @@ netsnmp_sockaddr_in6_2(struct sockaddr_in6 *addr,
                         if (portno <= 0)
                             portno = SNMP_PORT;
                         addr->sin6_port = htons(portno);
-#ifdef HAVE_IF_NAMETOINDEX
+#if defined(HAVE_IF_NAMETOINDEX) && defined(HAVE_STRUCT_SOCKADDR_IN6_SIN6_SCOPE_ID)
                         addr->sin6_scope_id = if_index;
 #endif
                         goto resolved;
@@ -491,7 +490,7 @@ netsnmp_sockaddr_in6_2(struct sockaddr_in6 *addr,
                             "IPv6 address with port suffix :%d\n",
                             atoi(cp + 1)));
                 addr->sin6_port = htons(atoi(cp + 1));
-#ifdef HAVE_IF_NAMETOINDEX
+#if defined(HAVE_IF_NAMETOINDEX) && defined(HAVE_STRUCT_SOCKADDR_IN6_SIN6_SCOPE_ID)
                 addr->sin6_scope_id = if_index;
 #endif
                 goto resolved;
@@ -536,6 +535,13 @@ netsnmp_sockaddr_in6_2(struct sockaddr_in6 *addr,
              * Fall through.  
              */
         }
+
+        if (peername[0] == '\0') {
+          DEBUGMSGTL(("netsnmp_sockaddr_in6", "empty hostname\n"));
+          free(peername);
+          return 0;
+        }
+
 #if HAVE_GETADDRINFO
         memset(&hint, 0, sizeof hint);
         hint.ai_flags = 0;
@@ -545,8 +551,13 @@ netsnmp_sockaddr_in6_2(struct sockaddr_in6 *addr,
 
         err = getaddrinfo(peername, NULL, &hint, &addrs);
         if (err != 0) {
-            snmp_log(LOG_ERR, "getaddrinfo: %s %s\n", peername,
+#if HAVE_GAI_STRERROR
+            snmp_log(LOG_ERR, "getaddrinfo(\"%s\", NULL, ...): %s\n", peername,
                      gai_strerror(err));
+#else
+            snmp_log(LOG_ERR, "getaddrinfo(\"%s\", NULL, ...): (error %d)\n",
+                     peername, err);
+#endif
             free(peername);
             return 0;
         }
@@ -943,7 +954,7 @@ masked_address_are_equal(int af, struct sockaddr_storage *from,
 #define EXAMPLE_COMMUNITY     "COMMUNITY"
 
 typedef struct _com2Sec6Entry {
-    char            community[VACMSTRINGLEN];
+    char            community[COMMUNITY_MAX_LEN];
     struct sockaddr_in6 network;
     struct sockaddr_in6 mask;
     char            secName[VACMSTRINGLEN];
@@ -979,7 +990,7 @@ netsnmp_udp6_parse_security(const char *token, char *param)
 {
     char            secName[VACMSTRINGLEN];
     char            contextName[VACMSTRINGLEN];
-    char            community[VACMSTRINGLEN];
+    char            community[COMMUNITY_MAX_LEN];
     char            source[IPV6_STRING_LEN];
     char           *cp = NULL, *strnetwork = NULL, *strmask = NULL;
     com2Sec6Entry  *e = NULL;
@@ -1034,7 +1045,7 @@ netsnmp_udp6_parse_security(const char *token, char *param)
             == 0) {
         config_perror("example config COMMUNITY not properly configured");
         return;
-    } else if (strlen(community) > (VACMSTRINGLEN - 1)) {
+    } else if (strlen(community) > (COMMUNITY_MAX_LEN - 1)) {
         config_perror("community name too long");
         return;
     }
@@ -1363,7 +1374,7 @@ netsnmp_udp6_ctor(void)
     udp6Domain.name_length = sizeof(netsnmp_UDPIPv6Domain) / sizeof(oid);
     udp6Domain.f_create_from_tstring_new = netsnmp_udp6_create_tstring;
     udp6Domain.f_create_from_ostring = netsnmp_udp6_create_ostring;
-    udp6Domain.prefix = calloc(5, sizeof(char *));
+    udp6Domain.prefix = (const char**)calloc(5, sizeof(char *));
     udp6Domain.prefix[0] = "udp6";
     udp6Domain.prefix[1] = "ipv6";
     udp6Domain.prefix[2] = "udpv6";
