@@ -134,7 +134,7 @@ netsnmp_unix_recv(netsnmp_transport *t, void *buf, int size,
             return -1;
         };
         while (rc < 0) {
-            rc = recv(t->sock, buf, size, 0);
+            rc = recvfrom(t->sock, buf, size, 0, NULL, 0);
             if (rc < 0 && errno != EINTR) {
                 DEBUGMSGTL(("netsnmp_unix", "recv fd %d err %d (\"%s\")\n",
                             t->sock, errno, strerror(errno)));
@@ -160,7 +160,7 @@ netsnmp_unix_send(netsnmp_transport *t, void *buf, int size,
         DEBUGMSGTL(("netsnmp_unix", "send %d bytes to %p on fd %d\n",
                     size, buf, t->sock));
         while (rc < 0) {
-            rc = send(t->sock, buf, size, 0);
+            rc = sendto(t->sock, buf, size, 0, NULL, 0);
             if (rc < 0 && errno != EINTR) {
                 break;
             }
@@ -439,7 +439,7 @@ netsnmp_unix_create_ostring(const u_char * o, size_t o_len, int local)
     if (o_len > 0 && o_len < (sizeof(addr.sun_path) - 1)) {
         addr.sun_family = AF_UNIX;
         memset(addr.sun_path, 0, sizeof(addr.sun_path));
-        strncpy(addr.sun_path, (char *)o, o_len);
+        strncpy(addr.sun_path, (const char *)o, o_len);
         return netsnmp_unix_transport(&addr, local);
     } else {
         if (o_len > 0) {
@@ -470,7 +470,7 @@ netsnmp_unix_ctor(void)
 
 #define EXAMPLE_COMMUNITY "COMMUNITY"
 typedef struct _com2SecUnixEntry {
-    char            community[VACMSTRINGLEN];
+    char            community[COMMUNITY_MAX_LEN];
     char            sockpath[sizeof(struct sockaddr_un)];
     unsigned long   pathlen;
     char            secName[VACMSTRINGLEN];
@@ -555,7 +555,7 @@ netsnmp_unix_getSecName(void *opaque, int olength,
 void
 netsnmp_unix_parse_security(const char *token, char *param)
 {
-    char              secName[VACMSTRINGLEN + 1], community[VACMSTRINGLEN + 1];
+    char              secName[VACMSTRINGLEN + 1], community[COMMUNITY_MAX_LEN + 1];
     char              contextName[VACMSTRINGLEN + 1];
     char              sockpath[sizeof(struct sockaddr_un) + 1];
     com2SecUnixEntry *e = NULL;
@@ -593,7 +593,7 @@ netsnmp_unix_parse_security(const char *token, char *param)
         sockpath[0] = 0;
     }
 
-    param = copy_nword(param, community, VACMSTRINGLEN);
+    param = copy_nword(param, community, COMMUNITY_MAX_LEN);
     if (community[0] == '\0') {
         config_perror("missing COMMUNITY parameter\n");
         return;
@@ -602,7 +602,7 @@ netsnmp_unix_parse_security(const char *token, char *param)
                == 0) {
         config_perror("example config COMMUNITY not properly configured");
         return;
-    } else if (strlen(community) > (VACMSTRINGLEN - 1)) {
+    } else if (strlen(community) > (COMMUNITY_MAX_LEN - 1)) {
         config_perror("community name too long");
         return;
     }
